-------------------------------------------------------------
--	Filename:  CIC_INTERPOL3.VHD
-- Authors: Adam Kwiatkowski
--		  Alain Zarembowitch 	 
--	Version: 3
--	Date last modified: 5/27/08
-- Inheritance: 	CIC.VHD v1  9-12-05, CIC_Interpol.vhd 9-28-05
--
-- Objective: Cascaded Integrated Comb interpolation filter.
-- Observe limits listed in the interface for control parameters N,R,M and truncation.
-- Use R = 1 to bypass this filter.
-- Simplified version of the more versatile CIC_INTERPOL.VHD:
-- N = 2, M = 1, Output truncation = 2*log2(R) rounded up to the nearest integer,
-- output sampling frequency FFFFFFFF
-- The code is flexible (N,M,output sampling frequency can be changed then resynthesized).
--
-- Implementation: The precision of the CIC internal signals is extended to 64-bit
-- at the input. This limits the range of recommended control parameters so that
-- Bin + Nlog2(RM) < 65. 
--
-- For more details, see "CIC Filter Introduction", Matthew P. Donadio
-- and Xilinx LogicCore documentation on "Cascaded Integrated-Comb (CIC) Filter v1.0".
--
-- Rev 2 4/1/06 AZ
-- Increased gain so that the CIC_INTERPOL3 gain is exactly 1 for Interpolation 1,2,4,8,16,etc..
-- and GREATER THAN 1 for other interpolation factors. Reason: when gain is less than 1, it
-- may not be possible to reach the maximum output power.
-- Consequently, depending on the data rate selected, the modulator gain should be adjusted to prevent
-- saturation.
-- + finer output truncation (steps of 1 instead of 2 as previous).
--
-- Rev 3 5/27/08 AZ
-- commented out the interpolation NCO. Wasteful.
-- corrected bug on output saturation
-- added saturation detector test point
---------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

--  Uncomment the following lines to use the declarations that are
--  provided for instantiating Xilinx primitive components.
--library UNISIM;
--use UNISIM.VComponents.all;

entity CIC_INTERPOL3 is
    port ( 

	--// GLOBAL CLOCKS, RESET
   CLK : in std_logic;				-- Master clock for this FPGA
	SYNC_RESET: in std_logic;	
		-- reset all upon configuration change.. otherwise an annoying DC offset 
		-- may be present on the output and cause saturation.
	
	--// INPUT DATA STREAM
	DATA_IN: in std_logic_vector(11 downto 0);
		-- input samples, 12-bit precision
		-- read at rising edge of CLK when SAMPLE_CLK_IN = '1'.
		-- Format: 2's complement 

	--// CONTROL PARAMETERS
	R: in integer range 1 to 16384;
		-- Interpolation factor
		-- Set R = 1 to bypass.
		-- Otherwise, nominal range is 8 to 16384.
		-- Note: R can be in the range 2-7 as long as the number of bits to be truncated is less than R.

	--// OUTPUT DATA STREAM
	DATA_OUT: out std_logic_vector(11 downto 0);   
		-- Format: 2's complement
		-- The latency between output samples and input samples is variable (depends on N,R,M):
		-- 11 CLKs fixed + 1 CLK for each 8 bits of output truncation + 1 CLK for each remaining 1 bit of output truncation.
		-- For example: if N=4,M=1,R=7, TRUNCATE_OBITS = 12. The total latency is 16 CLKs.
	SATURATION: out std_logic
		-- '1' when interpolation circuit saturates. Reduce input signal level when this occurs.
		-- Important test point, as the effects of saturation (spectrum degradation) can be difficult to detect.
			);
end entity;

architecture behavioral of CIC_INTERPOL3 is
--------------------------------------------------------
--      COMPONENTS
--------------------------------------------------------

--------------------------------------------------------
--     SIGNALS
--------------------------------------------------------
-- Suffix _D indicates a one CLK delayed version of the net with the same name
-- Suffix _E indicates an extended precision version of the net with the same name
-- Suffix _N indicates an inverted version of the net with the same name
-- Suffic _INC indicates increment.
constant	N: integer range 1 to 8 := 2;
		-- number of stages
constant	M: integer range 1 to 2 := 1;
		-- differential delay    
-- change commented out lines during simulation
-- for simulation purposes
--constant BOUT: integer  := 31;	-- limit range to 32 bits for simulation purposes only. compile with larger precision
constant BOUT: integer  := 40;  -- 12-bit input + 2*log2(R(16384 max))	
	-- computation precision to accommodate bit growth (total array length BOUT+1)
constant BIN: integer  := 11;	
	-- computation precision to accommodate bit growth (total array length BIN+1)
signal TRUNCATE_OBITS: std_logic_vector(5 downto 0);
		-- number of least significant bits to be truncated at the output. 
		-- Must be set to N*log2(RM), which is the bit growth during the CIC
		-- calculation due to the gain G = (RM)^N.
		-- As this CIC module is designed to be flexible, it is generally more efficient
		-- to compute this field by the application (i.e. outside this module) 
		-- since the parameters N,R,M are generally known and fixed at the time of 
		-- synthesis. 
		-- Valid range 0 - BOUT (here 31)

signal DATA_IN_D: std_logic_vector(BOUT downto 0);
signal DATA_IN_D2: std_logic_vector(BOUT downto 0);
signal DATA_IN_D3: std_logic_vector(BOUT downto 0);

--// comb
signal SAMPLE1_CLK: std_logic;
signal DATA1: std_logic_vector(BOUT downto 0);
signal DATA1_D: std_logic_vector(BOUT downto 0);
signal DATA1_D2: std_logic_vector(BOUT downto 0);
signal SAMPLE2_CLK: std_logic;
signal DATA2: std_logic_vector(BOUT downto 0);
signal DATA2_D: std_logic_vector(BOUT downto 0);
signal DATA2_D2: std_logic_vector(BOUT downto 0);
signal SAMPLE3_CLK: std_logic;
signal DATA3: std_logic_vector(BOUT downto 0);
signal DATA3_D: std_logic_vector(BOUT downto 0);
signal DATA3_D2: std_logic_vector(BOUT downto 0);
signal SAMPLE4_CLK: std_logic;
signal DATA4: std_logic_vector(BOUT downto 0);
signal DATA4_D: std_logic_vector(BOUT downto 0);
signal DATA4_D2: std_logic_vector(BOUT downto 0);
signal SAMPLE5_CLK: std_logic;
signal DATA5: std_logic_vector(BOUT downto 0);
signal DATA5_D: std_logic_vector(BOUT downto 0);
signal DATA5_D2: std_logic_vector(BOUT downto 0);
signal SAMPLE6_CLK: std_logic;
signal DATA6: std_logic_vector(BOUT downto 0);
signal DATA6_D: std_logic_vector(BOUT downto 0);
signal DATA6_D2: std_logic_vector(BOUT downto 0);
signal SAMPLE7_CLK: std_logic;
signal DATA7: std_logic_vector(BOUT downto 0);
signal DATA7_D: std_logic_vector(BOUT downto 0);
signal DATA7_D2: std_logic_vector(BOUT downto 0);
signal SAMPLE8_CLK: std_logic;
signal DATA8: std_logic_vector(BOUT downto 0);
signal SAMPLE9_CLK: std_logic;
signal DATA9: std_logic_vector(BOUT downto 0);

--// Interpolation
signal INTERPOLATION_COUNTER: std_logic_vector(13 downto 0);	
signal INTERPOLATION_COUNTER_INC: std_logic_vector(13 downto 0);
signal COMB_CLK: std_logic;
signal COMB_CLK_D: std_logic;

--// integrators
signal SAMPLE11_CLK: std_logic;
signal DATA11: std_logic_vector(BOUT downto 0);
signal SAMPLE12_CLK: std_logic;
signal DATA12: std_logic_vector(BOUT downto 0);
signal SAMPLE13_CLK: std_logic;
signal DATA13: std_logic_vector(BOUT downto 0);
signal SAMPLE14_CLK: std_logic;
signal DATA14: std_logic_vector(BOUT downto 0);
signal SAMPLE15_CLK: std_logic;
signal DATA15: std_logic_vector(BOUT downto 0);
signal SAMPLE16_CLK: std_logic;
signal DATA16: std_logic_vector(BOUT downto 0);
signal SAMPLE17_CLK: std_logic;
signal DATA17: std_logic_vector(BOUT downto 0);
signal SAMPLE18_CLK: std_logic;
signal DATA18: std_logic_vector(BOUT downto 0);
signal SAMPLE19_CLK: std_logic;
signal DATA19: std_logic_vector(BOUT downto 0);
signal SAMPLE19A_CLK: std_logic;
signal DATA19A: std_logic_vector(BOUT downto 0);

signal SAMPLE20_CLK: std_logic;
signal DATA20: std_logic_vector(BOUT downto 0);
--------------------------------------------------------
--      IMPLEMENTATION
--------------------------------------------------------

begin
-- compute the number of output bits to truncate
TRUNCATE_OBITS_GEN: process(SYNC_RESET, CLK, R)
begin
	if rising_edge(CLK) then
		if(SYNC_RESET = '1') then
			if(R = 1) then
				TRUNCATE_OBITS <= "000000";
			elsif(R = 2) then
				TRUNCATE_OBITS <= "000010";
			elsif(R = 3) then
				TRUNCATE_OBITS <= "000011";
			elsif(R < 6) then
				TRUNCATE_OBITS <= "000100";
			elsif(R < 8) then
				TRUNCATE_OBITS <= "000101";
			elsif(R < 12) then
				TRUNCATE_OBITS <= "000110";
			elsif(R < 16) then
				TRUNCATE_OBITS <= "000111";
			elsif(R < 23) then
				TRUNCATE_OBITS <= "001000";
			elsif(R < 32) then
				TRUNCATE_OBITS <= "001001";
			elsif(R < 46) then
				TRUNCATE_OBITS <= "001010";
			elsif(R < 64) then
				TRUNCATE_OBITS <= "001011";
			elsif(R < 91) then
				TRUNCATE_OBITS <= "001100";
			elsif(R < 128) then
				TRUNCATE_OBITS <= "001101";
			elsif(R < 182) then
				TRUNCATE_OBITS <= "001110";
			elsif(R < 256) then
				TRUNCATE_OBITS <= "001111";
			elsif(R < 363) then
				TRUNCATE_OBITS <= "010000";
			elsif(R < 512) then
				TRUNCATE_OBITS <= "010001";
			elsif(R < 725) then
				TRUNCATE_OBITS <= "010010";
			elsif(R < 1024) then
				TRUNCATE_OBITS <= "010011";
			elsif(R < 1449) then
				TRUNCATE_OBITS <= "010100";
			elsif(R < 2048) then
				TRUNCATE_OBITS <= "010101";
			elsif(R < 2897) then
				TRUNCATE_OBITS <= "010110";
			elsif(R < 4096) then
				TRUNCATE_OBITS <= "010111";
			elsif(R < 5793) then
				TRUNCATE_OBITS <= "011000";
			elsif(R < 8192) then
				TRUNCATE_OBITS <= "011001";
			elsif(R < 11586) then
				TRUNCATE_OBITS <= "011010";
			elsif(R < 16384) then
				TRUNCATE_OBITS <= "011011";
			else
				TRUNCATE_OBITS <= "011100";
			end if;
		end if;
	end if;
end process;


--------------------------------------------------------------------------
-- READ CLOCK NCO
--------------------------------------------------------------------------
--// interpolation counter
INTERPOLATION_COUNTER_INC <= INTERPOLATION_COUNTER + 1;

NCO_GEN: process(CLK)
begin
	if rising_edge(CLK) then
		if(SYNC_RESET = '1') then
			INTERPOLATION_COUNTER <= (others => '0');
			COMB_CLK <= '0';
		else 
			if(INTERPOLATION_COUNTER_INC = R) then
				-- modulo R counter
				INTERPOLATION_COUNTER <= (others => '0');
				COMB_CLK <= '1';
			else 
				INTERPOLATION_COUNTER <= INTERPOLATION_COUNTER_INC;
				COMB_CLK <= '0';
			end if;
		end if;
	end if;
end process;

--// Reclock input signal for best timing. 
-- Extend precision to 32-bits because of the bit growth.  
RECLOCK_001: process(CLK)
begin
	if rising_edge(CLK) then
		if(SYNC_RESET = '1') then
			COMB_CLK_D <= '0';
			DATA_IN_D <= (others => '0');
		else
			COMB_CLK_D <= COMB_CLK;
			if(COMB_CLK = '1') then
				DATA_IN_D(BIN downto 0) <= DATA_IN;
				-- precision extension
				DATA_IN_D(BOUT downto (BIN+1)) <= (others => DATA_IN(BIN));
			end if;	
		end if;
	end if;
end process;  
  
--// 8 comb stages
-- Comb stage 1 
COMB1: process(CLK)
begin
	if rising_edge(CLK) then
		if(SYNC_RESET = '1') then
			SAMPLE1_CLK <= '0';
			DATA_IN_D2 <= (others => '0');
			DATA_IN_D3 <= (others => '0');
			DATA1 <= (others => '0');
		else
			if(COMB_CLK_D = '1') then
				SAMPLE1_CLK <= '1';
				DATA_IN_D2 <= DATA_IN_D;
				DATA_IN_D3 <= DATA_IN_D2;
				if(M = 1) then
					DATA1 <= DATA_IN_D - DATA_IN_D2;
				elsif(M = 2) then
					DATA1 <= DATA_IN_D - DATA_IN_D3;
				end if;
			else
				SAMPLE1_CLK <= '0';
			end if;
		end if;
	end if;
end process;

-- Comb stage 2 
COMB2: process(CLK)
begin
	if rising_edge(CLK) then
		if(SYNC_RESET = '1') then
			SAMPLE2_CLK <= '0';
			DATA1_D <= (others => '0');
			DATA1_D2 <= (others => '0');
			DATA2 <= (others => '0');
		else
			SAMPLE2_CLK <= SAMPLE1_CLK;

			if(SAMPLE1_CLK = '1') then
				DATA1_D <= DATA1;
				DATA1_D2 <= DATA1_D;

				if(M = 1) then
					DATA2 <= DATA1 - DATA1_D;
				elsif(M = 2) then
					DATA2 <= DATA1 - DATA1_D2;
				end if;
			end if;
		end if;
	end if;
end process;

-- Comb stage 3 
COMB3: process(CLK)
begin
	if rising_edge(CLK) then
		if(SYNC_RESET = '1') then
			SAMPLE3_CLK <= '0';
			DATA2_D <= (others => '0');
			DATA2_D2 <= (others => '0');
			DATA3 <= (others => '0');
		else
			SAMPLE3_CLK <= SAMPLE2_CLK;

			if(SAMPLE2_CLK = '1') then
				DATA2_D <= DATA2;
				DATA2_D2 <= DATA2_D;

				if(M = 1) then
					DATA3 <= DATA2 - DATA2_D;
				elsif(M = 2) then
					DATA3 <= DATA2 - DATA2_D2;
				end if;
			end if;
		end if;
	end if;
end process;

-- Comb stage 4 
COMB4: process(CLK)
begin
	if rising_edge(CLK) then
		if(SYNC_RESET = '1') then
			SAMPLE4_CLK <= '0';
			DATA3_D <= (others => '0');
			DATA3_D2 <= (others => '0');
			DATA4 <= (others => '0');
		else
			SAMPLE4_CLK <= SAMPLE3_CLK;

			if(SAMPLE3_CLK = '1') then
				DATA3_D <= DATA3;
				DATA3_D2 <= DATA3_D;

				if(M = 1) then
					DATA4 <= DATA3 - DATA3_D;
				elsif(M = 2) then
					DATA4 <= DATA3 - DATA3_D2;
				end if;
			end if;
		end if;
	end if;
end process;

-- Comb stage 5 
COMB5: process(CLK)
begin
	if rising_edge(CLK) then
		if(SYNC_RESET = '1') then
			SAMPLE5_CLK <= '0';
			DATA4_D <= (others => '0');
			DATA4_D2 <= (others => '0');
			DATA5 <= (others => '0');
		else
			SAMPLE5_CLK <= SAMPLE4_CLK;

			if(SAMPLE4_CLK = '1') then
				DATA4_D <= DATA4;
				DATA4_D2 <= DATA4_D;

				if(M = 1) then
					DATA5 <= DATA4 - DATA4_D;
				elsif(M = 2) then
					DATA5 <= DATA4 - DATA4_D2;
				end if;
			end if;
		end if;
	end if;
end process;

-- Comb stage 6 
COMB6: process(CLK)
begin
	if rising_edge(CLK) then
		if(SYNC_RESET = '1') then
			SAMPLE6_CLK <= '0';
			DATA5_D <= (others => '0');
			DATA5_D2 <= (others => '0');
			DATA6 <= (others => '0');
		else
			SAMPLE6_CLK <= SAMPLE5_CLK;

			if(SAMPLE5_CLK = '1') then
				DATA5_D <= DATA5;
				DATA5_D2 <= DATA5_D;

				if(M = 1) then
					DATA6 <= DATA5 - DATA5_D;
				elsif(M = 2) then
					DATA6 <= DATA5 - DATA5_D2;
				end if;
			end if;
		end if;
	end if;
end process;

-- Comb stage 7 
COMB7: process(CLK)
begin
	if rising_edge(CLK) then
		if(SYNC_RESET = '1') then
			SAMPLE7_CLK <= '0';
			DATA6_D <= (others => '0');
			DATA6_D2 <= (others => '0');
			DATA7 <= (others => '0');
		else
			SAMPLE7_CLK <= SAMPLE6_CLK;

			if(SAMPLE6_CLK = '1') then
				DATA6_D <= DATA6;
				DATA6_D2 <= DATA6_D;

				if(M = 1) then
					DATA7 <= DATA6 - DATA6_D;
				elsif(M = 2) then
					DATA7 <= DATA6 - DATA6_D2;
				end if;
			end if;
		end if;
	end if;
end process;

-- Comb stage 8 
COMB8: process(CLK)
begin
	if rising_edge(CLK) then
		if(SYNC_RESET = '1') then
			SAMPLE8_CLK <= '0';
			DATA7_D <= (others => '0');
			DATA7_D2 <= (others => '0');
			DATA8 <= (others => '0');
		else
			SAMPLE8_CLK <= SAMPLE7_CLK;

			if(SAMPLE7_CLK = '1') then
				DATA7_D <= DATA7;
				DATA7_D2 <= DATA7_D;

				if(M = 1) then
					DATA8 <= DATA7 - DATA7_D;
				elsif(M = 2) then
					DATA8 <= DATA7 - DATA7_D2;
				end if;
			end if;
		end if;
	end if;
end process;

 
--// Select number of comb stages N 
STAGES_SEL_001: process(SAMPLE1_CLK, SAMPLE2_CLK, SAMPLE3_CLK, SAMPLE4_CLK, SAMPLE5_CLK, SAMPLE6_CLK,
				SAMPLE7_CLK, SAMPLE8_CLK, DATA1, DATA2, DATA3, DATA4, DATA5, DATA6, DATA7, DATA8)
begin
   case (N) is 
      when 1 =>
		SAMPLE9_CLK <= SAMPLE1_CLK;
		DATA9 <= DATA1;
      when 2 =>
      	SAMPLE9_CLK <= SAMPLE2_CLK;
		DATA9 <= DATA2;
      when 3 =>
		SAMPLE9_CLK <= SAMPLE3_CLK;
		DATA9 <= DATA3;
      when 4 =>
		SAMPLE9_CLK <= SAMPLE4_CLK;
		DATA9 <= DATA4;
      when 5 =>
		SAMPLE9_CLK <= SAMPLE5_CLK;
		DATA9 <= DATA5;
      when 6 =>
		SAMPLE9_CLK <= SAMPLE6_CLK;
		DATA9 <= DATA6;
      when 7 =>
		SAMPLE9_CLK <= SAMPLE7_CLK;
		DATA9 <= DATA7;
      when others =>
		SAMPLE9_CLK <= SAMPLE8_CLK;
		DATA9 <= DATA8;
   end case;
end process;      

 

--// Rate Expansion

--// 8 Integrator stages
-- Integrator stage 1
INTEGRATOR_001: process(CLK)
begin
	if rising_edge(CLK) then
		if(SYNC_RESET = '1') then
			SAMPLE11_CLK <= '0';
			DATA11 <= (others => '0');
		else
			SAMPLE11_CLK <= '1';
			DATA11 <= DATA11 + DATA9;
		end if;
	end if;
end process;

-- Integrator stage 2
INTEGRATOR_002: process(CLK)
begin
	if rising_edge(CLK) then
		if(SYNC_RESET = '1') then
			SAMPLE12_CLK <= '0';
			DATA12 <= (others => '0');
		else
			SAMPLE12_CLK <= SAMPLE11_CLK;

			if(SAMPLE11_CLK = '1') then
				DATA12 <= DATA12 + DATA11;
			end if;
		end if;
	end if;
end process;

-- Integrator stage 3
INTEGRATOR_003: process(CLK)
begin
	if rising_edge(CLK) then
		if(SYNC_RESET = '1') then
			SAMPLE13_CLK <= '0';
			DATA13 <= (others => '0');
		else
			SAMPLE13_CLK <= SAMPLE12_CLK;

			if(SAMPLE12_CLK = '1') then
				DATA13 <= DATA13 + DATA12;
			end if;
		end if;
	end if;
end process;

-- Integrator stage 4
INTEGRATOR_004: process(CLK)
begin
	if rising_edge(CLK) then
		if(SYNC_RESET = '1') then
			SAMPLE14_CLK <= '0';
			DATA14 <= (others => '0');
		else
			SAMPLE14_CLK <= SAMPLE13_CLK;

			if(SAMPLE13_CLK = '1') then
				DATA14 <= DATA14 + DATA13;
			end if;
		end if;
	end if;
end process;

-- Integrator stage 5
INTEGRATOR_005: process(CLK)
begin
	if rising_edge(CLK) then
		if(SYNC_RESET = '1') then
			SAMPLE15_CLK <= '0';
			DATA15 <= (others => '0');
		else
			SAMPLE15_CLK <= SAMPLE14_CLK;

			if(SAMPLE14_CLK = '1') then
				DATA15 <= DATA15 + DATA14;
			end if;
		end if;
	end if;
end process;

-- Integrator stage 6
INTEGRATOR_006: process(CLK)
begin
	if rising_edge(CLK) then
		if(SYNC_RESET = '1') then
			SAMPLE16_CLK <= '0';
			DATA16 <= (others => '0');
		else
			SAMPLE16_CLK <= SAMPLE15_CLK;

			if(SAMPLE15_CLK = '1') then
				DATA16 <= DATA16 + DATA15;
			end if;
		end if;
	end if;
end process;

-- Integrator stage 7
INTEGRATOR_007: process(CLK)
begin
	if rising_edge(CLK) then
		if(SYNC_RESET = '1') then
			SAMPLE17_CLK <= '0';
			DATA17 <= (others => '0');
		else
			SAMPLE17_CLK <= SAMPLE16_CLK;

			if(SAMPLE16_CLK = '1') then
				DATA17 <= DATA17 + DATA16;
			end if;
		end if;
	end if;
end process;

-- Integrator stage 8
INTEGRATOR_008: process(CLK)
begin
	if rising_edge(CLK) then
		if(SYNC_RESET = '1') then
			SAMPLE18_CLK <= '0';
			DATA18 <= (others => '0');
		else
			SAMPLE18_CLK <= SAMPLE17_CLK;

			if(SAMPLE17_CLK = '1') then
				DATA18 <= DATA18 + DATA17;
			end if;
		end if;
	end if;
end process;



--// Select number of integrator stages N
STAGES_SEL_002: process(SAMPLE11_CLK, SAMPLE12_CLK, SAMPLE13_CLK, SAMPLE14_CLK, SAMPLE15_CLK, SAMPLE16_CLK,
				SAMPLE17_CLK, SAMPLE18_CLK, DATA11, DATA12, DATA13, DATA14, DATA15, DATA16, DATA17, DATA18)
begin
   case (N) is 
      when 1 =>
         SAMPLE19_CLK <= SAMPLE11_CLK;
			DATA19 <= DATA11;
      when 2 =>
         SAMPLE19_CLK <= SAMPLE12_CLK;
			DATA19 <= DATA12;
      when 3 =>
         SAMPLE19_CLK <= SAMPLE13_CLK;
			DATA19 <= DATA13;
      when 4 =>
         SAMPLE19_CLK <= SAMPLE14_CLK;
			DATA19 <= DATA14;
      when 5 =>
         SAMPLE19_CLK <= SAMPLE15_CLK;
			DATA19 <= DATA15;
      when 6 =>
         SAMPLE19_CLK <= SAMPLE16_CLK;
			DATA19 <= DATA16;
      when 7 =>
         SAMPLE19_CLK <= SAMPLE17_CLK;
			DATA19 <= DATA17;
      when others =>
         SAMPLE19_CLK <= SAMPLE18_CLK;
			DATA19 <= DATA18;
   end case;
end process;

--// Truncate signal
-- Shift in two steps. Coarse and Fine.
COARSE_SHIFT_001: process(CLK, TRUNCATE_OBITS)
begin
	if rising_edge(CLK) then
		if(SYNC_RESET = '1') then
			SAMPLE19A_CLK <= '0';
			DATA19A <= (others => '0');
		else
			SAMPLE19A_CLK <= SAMPLE19_CLK;		

			if(SAMPLE19_CLK = '1')  then
				if(TRUNCATE_OBITS(5 downto 3) = 0) then
					DATA19A <= DATA19;
				elsif(TRUNCATE_OBITS(5 downto 3) = 1) then
					DATA19A((BOUT-8) downto 0) <= DATA19(BOUT downto 8);
					DATA19A(BOUT downto (BOUT-7)) <= (others => DATA19(BOUT)); -- sign extension
				elsif(TRUNCATE_OBITS(5 downto 3) = 2) then
					DATA19A((BOUT-16) downto 0) <= DATA19(BOUT downto 16);
					DATA19A(BOUT downto (BOUT-15)) <= (others => DATA19(BOUT)); -- sign extension
				else
				--elsif(TRUNCATE_OBITS(5 downto 3) = 3) then
					DATA19A((BOUT-24) downto 0) <= DATA19(BOUT downto 24);
					DATA19A(BOUT downto (BOUT-23)) <= (others => DATA19(BOUT)); -- sign extension
	--			elsif(TRUNCATE_OBITS(5 downto 3) = 4) then
	--				DATA19A((BOUT-32) downto 0) <= DATA19(BOUT downto 32);
	--				DATA19A(BOUT downto (BOUT-31)) <= (others => DATA19(BOUT)); -- sign extension
	--			elsif(TRUNCATE_OBITS(5 downto 3) = 5) then
	--				DATA19A((BOUT-40) downto 0) <= DATA19(BOUT downto 40);
	--				DATA19A(BOUT downto (BOUT-39)) <= (others => DATA19(BOUT)); -- sign extension
	--			elsif(TRUNCATE_OBITS(5 downto 3) = 6) then
	--				DATA19A((BOUT-48) downto 0) <= DATA19(BOUT downto 48);
	--				DATA19A(BOUT downto (BOUT-47)) <= (others => DATA19(BOUT)); -- sign extension
	--			else
	--			--elsif(TRUNCATE_OBITS(5 downto 3) = 7) then
	--				DATA19A((BOUT-56) downto 0) <= DATA19(BOUT downto 56);
	--				DATA19A(BOUT downto (BOUT-55)) <= (others => DATA19(BOUT)); -- sign extension
				end if;
			end if;
		end if;
	end if;
end process;

FINE_SHIFT_001: process(CLK, TRUNCATE_OBITS)
begin
	if rising_edge(CLK) then
		if(SYNC_RESET = '1') then
			SAMPLE20_CLK <= '0';
			DATA20 <= (others => '0');
		else
			SAMPLE20_CLK <= SAMPLE19A_CLK;		

			if(SAMPLE19_CLK = '1')  then
				if(TRUNCATE_OBITS(2 downto 0) = 0) then
					DATA20 <= DATA19A;
				elsif(TRUNCATE_OBITS(2 downto 0) = 1) then
					DATA20((BOUT-1) downto 0) <= DATA19A(BOUT downto 1);
					DATA20(BOUT downto (BOUT-0)) <= (others => DATA19A(BOUT)); -- sign extension
				elsif(TRUNCATE_OBITS(2 downto 0) = 2) then
					DATA20((BOUT-2) downto 0) <= DATA19A(BOUT downto 2);
					DATA20(BOUT downto (BOUT-1)) <= (others => DATA19A(BOUT)); -- sign extension
				elsif(TRUNCATE_OBITS(2 downto 0) = 3) then
					DATA20((BOUT-3) downto 0) <= DATA19A(BOUT downto 3);
					DATA20(BOUT downto (BOUT-2)) <= (others => DATA19A(BOUT)); -- sign extension
				elsif(TRUNCATE_OBITS(2 downto 0) = 4) then
					DATA20((BOUT-4) downto 0) <= DATA19A(BOUT downto 4);
					DATA20(BOUT downto (BOUT-3)) <= (others => DATA19A(BOUT)); -- sign extension
				elsif(TRUNCATE_OBITS(2 downto 0) = 5) then
					DATA20((BOUT-5) downto 0) <= DATA19A(BOUT downto 5);
					DATA20(BOUT downto (BOUT-4)) <= (others => DATA19A(BOUT)); -- sign extension
				elsif(TRUNCATE_OBITS(2 downto 0) = 6) then
					DATA20((BOUT-6) downto 0) <= DATA19A(BOUT downto 6);
					DATA20(BOUT downto (BOUT-5)) <= (others => DATA19A(BOUT)); -- sign extension
				else
	--			--elsif(TRUNCATE_OBITS(2 downto 0) = 7) then
					DATA20((BOUT-7) downto 0) <= DATA19A(BOUT downto 7);
					DATA20(BOUT downto (BOUT-6)) <= (others => DATA19A(BOUT)); -- sign extension
				end if;
			end if;
		end if;
	end if;
end process;


-- reclock output. Wait until shift/truncate process is complete to send the
-- output sample.
RECLOCK_003: process(CLK)
begin
	if rising_edge(CLK) then
		if(SYNC_RESET = '1') then
			DATA_OUT <= (others => '0');
		else
			if(R = 1) then
				-- bypass this stage
				DATA_OUT <= DATA_IN;
			else
				-- Get the output sample after shifting.
				-- clamp before truncating MSbs
				if(DATA20(BOUT) = '0') and (DATA20((BOUT-1) downto 11) /= 0) then
					-- positive saturation after clamping
					DATA_OUT <= "011111111111";     
					SATURATION <= '1';
				elsif(DATA20(BOUT) = '1') and ((not DATA20((BOUT-1) downto 11)) /= 0) then
					-- negative saturation after clamping
					DATA_OUT <= "100000000000";
					SATURATION <= '1';
				else
					-- just truncate MSBs
					DATA_OUT <= DATA20(11 downto 0);
					SATURATION <= '0';
				end if;
			end if;
		end if;
	end if;
end process;


end Behavioral;
