-------------------------------------------------------------
-- Filename:  COM3011.VHD
-- Authors: 
--		Alain Zarembowitch / MSS
-- Version: Rev 4
-- Last modified: 6/2/11
-- Inheritance: 	COM-1600
--
-- description:  Top level for COM-3011 template.
-- Framework to start developing new applications.
-- Template includes interface with ARM Cortex M3 microcontroller for the user to set 
-- control registers (REG100 through REGy) and read  (SREG0 - SREGy) status registers. 
--
-- Rev 1 4/10/11 AZ
-- Added frequency calibration for ADC_CLK
-- removed ComScope test code.
-- 
-- Rev 2 4/24/11 AZ
-- Added independent gain control and output level reporting after the LPF
-- so that the ARM processor can close the 2nd AGC.
--
-- Rev 3 5/23/11 AZ
-- Re-wrote 40 MHz reclocking to eliminate glitches
-- latch scaling factor and CIC decimation factor
--
-- Rev 4 6/2/11 AZ
-- Made ADC sampling clock available at all times to pin A13
---------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
Library UNISIM;
use UNISIM.vcomponents.all;

entity COM3011 is
    Port ( 
		CLK_REF: in std_logic;
			-- 20 MHz frequency reference (automatic selection internal crystal oscillator
			-- or locked onto an external 10/20 MHz frequency reference)
			-- The same frequency reference is also used by RF frequency synthesizer.
			
		--// ADC interface
		ADC_CLK: inout std_logic;	
			-- IN when using the external 100 MHz oscillator (good phase noise, a bit of frequency drift)
			-- OUT when the FPGA generate the sampling clock, either using a PLL (beware of the poor phase noise)
			-- or when using an external sampling clock. In this case, make sure the external 100 MHz 
			-- oscillator is disabled (INT_OSC_EN = '0');
		ADC_DATA_IN: in std_logic_vector(13 downto 0);	-- ADC data, use MSbs, 2's complement
		ADC_OR: in std_logic;	-- out-of-range
		ADC_PWDN: out std_logic;	-- power down, active high

		--// Right connector, 40 pin 2mmx20 male 
		-- See schematics
		CLK_OUT: out std_logic;
		OUTPUT: inout std_logic_vector(30 downto 0);
		EXT_ADC_CLK: in std_logic;
		
		--// Others
		PLL_STROBE: in std_logic;
		INT_OSC_EN: out std_logic;
		
		--// Monitoring & Control: ARM Cortex M3 microcontroller interface
		-- asynchronous bus (shared with NAND flash)
		-- set address at the rising edge of UC_WEB_IN when UC_CSIB_IN = '0' and UC_ALE_IN = '1'
		-- write data the rising edge of UC_WEB_IN when UC_CSIB_IN = '0'
		-- read data when  UC_CSIB_IN = '0' and UC_REB_IN = '0'
		UC_CSIB_IN: in std_logic;  -- chip select. active low.
		UC_ALE_IN: in std_logic;  -- address latch enable. active high.
		UC_REB_IN: in std_logic;  -- read enable #
		UC_WEB_IN: in std_logic;	-- write enable #
		UC_AD: inout std_logic_vector(7 downto 0)  -- shared address/data bus

		  
			  );
end entity;

architecture Behavioral of COM3011 is
--------------------------------------------------------
--      COMPONENTS
--------------------------------------------------------
	COMPONENT CLKGEN_PLL
	PORT(
		CLKREF_IN : IN std_logic;          
		CLKOUT0G : OUT std_logic;
		CLKOUT2G : OUT std_logic;
		LOCKED_PLL : OUT std_logic
		);
	END COMPONENT;

	COMPONENT DIGITAL_DC2
	PORT(
		CLK : IN std_logic;
		DATA_I_IN : IN std_logic_vector(11 downto 0);
		DATA_Q_IN : IN std_logic_vector(11 downto 0);
		SAMPLE_CLK_IN : IN std_logic;
		PHASE_ROTATION : IN std_logic_vector(11 downto 0);          
		DATA_I_OUT : OUT std_logic_vector(11 downto 0);
		DATA_Q_OUT : OUT std_logic_vector(11 downto 0);
		SAMPLE_CLK_OUT : OUT std_logic;
		SATURATION : OUT std_logic
		);
	END COMPONENT;

	COMPONENT NCO32
	PORT(
		CLK : IN std_logic;
		SYNC_RESET : IN std_logic;
		NOMINAL_FREQUENCY : IN std_logic_vector(31 downto 0);
		PHASE_OFFSET : IN std_logic_vector(31 downto 0);
		PHASE_OFFSET_SAMPLE_CLK : IN std_logic;          
		PHASE : OUT std_logic_vector(31 downto 0)
		);
	END COMPONENT;

	COMPONENT CIC
	PORT(
		CLK : IN std_logic;
		SYNC_RESET : IN std_logic;
		DATA_IN : IN std_logic_vector(11 downto 0);
		SAMPLE_CLK_IN : IN std_logic;
		N : in integer range 1 to 8;
		R : in integer range 1 to 16384;
		M : in integer range 1 to 2;
		TRUNCATE_OBITS : in integer range 0 to 63;         
		DATA_OUT : OUT std_logic_vector(11 downto 0);
		SAMPLE_CLK_OUT : OUT std_logic
		);
	END COMPONENT;

	COMPONENT RESAMPLING2
	PORT(
		CLK : IN std_logic;
		DATA_I_IN : IN std_logic_vector(11 downto 0);
		DATA_Q_IN : IN std_logic_vector(11 downto 0);
		SAMPLE_CLK_IN : IN std_logic;
		NOMINAL_RESAMPLING_RATE : IN std_logic_vector(31 downto 0);
		NCO_OFFSET : IN std_logic_vector(31 downto 0);
		NCO_OFFSET_SAMPLE_CLK : IN std_logic;          
		DATA_I_OUT : OUT std_logic_vector(11 downto 0);
		DATA_Q_OUT : OUT std_logic_vector(11 downto 0);
		SAMPLE_CLK_OUT : OUT std_logic
		);
	END COMPONENT;

	COMPONENT FIRHALFBAND3
	PORT(
		CLK : IN std_logic;
		SAMPLE_CLK_IN : IN std_logic;
		DATA_IN : IN std_logic_vector(13 downto 0);          
		SAMPLE_CLK_OUT : OUT std_logic;
		DATA_OUT : OUT std_logic_vector(13 downto 0)
		);
	END COMPONENT;

	COMPONENT CIC_INTERPOL3
	PORT(
		CLK : IN std_logic;
		SYNC_RESET : IN std_logic;
		DATA_IN : IN std_logic_vector(11 downto 0);
		R : in integer range 1 to 16384;
		DATA_OUT : OUT std_logic_vector(11 downto 0);
		SATURATION : OUT std_logic
		);
	END COMPONENT;

	component MULT18X18SIGNED is port( 
		A	: in STD_LOGIC_VECTOR (17 downto 0);
		 B	: in STD_LOGIC_VECTOR (17 downto 0);
		P : out STD_LOGIC_VECTOR (35 downto 0) );
	end component;

	COMPONENT COMSCOPE
	generic (
		-- select features as needed: number of traces, widths
		-- One must re-synthesize after changing these choices.
		-- enable individual traces. '1' for enable, '0' for disable
		TRACE1_ENABLED: in std_logic;	
		TRACE2_ENABLED: in std_logic;	
		TRACE3_ENABLED: in std_logic;	
		TRACE4_ENABLED: in std_logic
	);	
	PORT(
		CLK : IN std_logic;
		ASYNC_RESET : IN std_logic;
		SIGNED_REPRESENTATION_U_SIGNED_N : IN std_logic;
		REG237 : IN std_logic_vector(7 downto 0);
		REG238 : IN std_logic_vector(7 downto 0);
		REG239 : IN std_logic_vector(7 downto 0);
		REG240 : IN std_logic_vector(7 downto 0);
		REG241 : IN std_logic_vector(7 downto 0);
		REG242 : IN std_logic_vector(7 downto 0);
		REG243 : IN std_logic_vector(7 downto 0);
		REG244 : IN std_logic_vector(7 downto 0);
		REG245 : IN std_logic_vector(7 downto 0);
		REG246 : IN std_logic_vector(7 downto 0);
		REG247 : IN std_logic_vector(7 downto 0);
		REG248 : IN std_logic_vector(7 downto 0);
		REG249 : IN std_logic_vector(7 downto 0);
		SIGNAL_1_BIT_1_1 : IN std_logic;
		SIGNAL_1_BIT_SAMPLE_CLK_1_1 : IN std_logic;
		SIGNAL_1_BIT_1_2 : IN std_logic;
		SIGNAL_1_BIT_SAMPLE_CLK_1_2 : IN std_logic;
		SIGNAL_1_BIT_1_3 : IN std_logic;
		SIGNAL_1_BIT_SAMPLE_CLK_1_3 : IN std_logic;
		SIGNAL_1_BIT_1_4 : IN std_logic;
		SIGNAL_1_BIT_SAMPLE_CLK_1_4 : IN std_logic;
		SIGNAL_2_BIT_1_1 : IN std_logic_vector(1 downto 0);
		SIGNAL_2_BIT_SAMPLE_CLK_1_1 : IN std_logic;
		SIGNAL_2_BIT_1_2 : IN std_logic_vector(1 downto 0);
		SIGNAL_2_BIT_SAMPLE_CLK_1_2 : IN std_logic;
		SIGNAL_2_BIT_1_3 : IN std_logic_vector(1 downto 0);
		SIGNAL_2_BIT_SAMPLE_CLK_1_3 : IN std_logic;
		SIGNAL_2_BIT_1_4 : IN std_logic_vector(1 downto 0);
		SIGNAL_2_BIT_SAMPLE_CLK_1_4 : IN std_logic;
		SIGNAL_4_BIT_1_1 : IN std_logic_vector(3 downto 0);
		SIGNAL_4_BIT_SAMPLE_CLK_1_1 : IN std_logic;
		SIGNAL_4_BIT_1_2 : IN std_logic_vector(3 downto 0);
		SIGNAL_4_BIT_SAMPLE_CLK_1_2 : IN std_logic;
		SIGNAL_4_BIT_1_3 : IN std_logic_vector(3 downto 0);
		SIGNAL_4_BIT_SAMPLE_CLK_1_3 : IN std_logic;
		SIGNAL_4_BIT_1_4 : IN std_logic_vector(3 downto 0);
		SIGNAL_4_BIT_SAMPLE_CLK_1_4 : IN std_logic;
		SIGNAL_8_BIT_1_1 : IN std_logic_vector(7 downto 0);
		SIGNAL_8_BIT_SAMPLE_CLK_1_1 : IN std_logic;
		SIGNAL_8_BIT_1_2 : IN std_logic_vector(7 downto 0);
		SIGNAL_8_BIT_SAMPLE_CLK_1_2 : IN std_logic;
		SIGNAL_8_BIT_1_3 : IN std_logic_vector(7 downto 0);
		SIGNAL_8_BIT_SAMPLE_CLK_1_3 : IN std_logic;
		SIGNAL_8_BIT_1_4 : IN std_logic_vector(7 downto 0);
		SIGNAL_8_BIT_SAMPLE_CLK_1_4 : IN std_logic;
		SIGNAL_16_BIT_1_1 : IN std_logic_vector(15 downto 0);
		SIGNAL_16_BIT_SAMPLE_CLK_1_1 : IN std_logic;
		SIGNAL_16_BIT_1_2 : IN std_logic_vector(15 downto 0);
		SIGNAL_16_BIT_SAMPLE_CLK_1_2 : IN std_logic;
		SIGNAL_16_BIT_1_3 : IN std_logic_vector(15 downto 0);
		SIGNAL_16_BIT_SAMPLE_CLK_1_3 : IN std_logic;
		SIGNAL_16_BIT_1_4 : IN std_logic_vector(15 downto 0);
		SIGNAL_16_BIT_SAMPLE_CLK_1_4 : IN std_logic;
		SIGNAL_1_BIT_2_1 : IN std_logic;
		SIGNAL_1_BIT_SAMPLE_CLK_2_1 : IN std_logic;
		SIGNAL_1_BIT_2_2 : IN std_logic;
		SIGNAL_1_BIT_SAMPLE_CLK_2_2 : IN std_logic;
		SIGNAL_1_BIT_2_3 : IN std_logic;
		SIGNAL_1_BIT_SAMPLE_CLK_2_3 : IN std_logic;
		SIGNAL_1_BIT_2_4 : IN std_logic;
		SIGNAL_1_BIT_SAMPLE_CLK_2_4 : IN std_logic;
		SIGNAL_2_BIT_2_1 : IN std_logic_vector(1 downto 0);
		SIGNAL_2_BIT_SAMPLE_CLK_2_1 : IN std_logic;
		SIGNAL_2_BIT_2_2 : IN std_logic_vector(1 downto 0);
		SIGNAL_2_BIT_SAMPLE_CLK_2_2 : IN std_logic;
		SIGNAL_2_BIT_2_3 : IN std_logic_vector(1 downto 0);
		SIGNAL_2_BIT_SAMPLE_CLK_2_3 : IN std_logic;
		SIGNAL_2_BIT_2_4 : IN std_logic_vector(1 downto 0);
		SIGNAL_2_BIT_SAMPLE_CLK_2_4 : IN std_logic;
		SIGNAL_4_BIT_2_1 : IN std_logic_vector(3 downto 0);
		SIGNAL_4_BIT_SAMPLE_CLK_2_1 : IN std_logic;
		SIGNAL_4_BIT_2_2 : IN std_logic_vector(3 downto 0);
		SIGNAL_4_BIT_SAMPLE_CLK_2_2 : IN std_logic;
		SIGNAL_4_BIT_2_3 : IN std_logic_vector(3 downto 0);
		SIGNAL_4_BIT_SAMPLE_CLK_2_3 : IN std_logic;
		SIGNAL_4_BIT_2_4 : IN std_logic_vector(3 downto 0);
		SIGNAL_4_BIT_SAMPLE_CLK_2_4 : IN std_logic;
		SIGNAL_8_BIT_2_1 : IN std_logic_vector(7 downto 0);
		SIGNAL_8_BIT_SAMPLE_CLK_2_1 : IN std_logic;
		SIGNAL_8_BIT_2_2 : IN std_logic_vector(7 downto 0);
		SIGNAL_8_BIT_SAMPLE_CLK_2_2 : IN std_logic;
		SIGNAL_8_BIT_2_3 : IN std_logic_vector(7 downto 0);
		SIGNAL_8_BIT_SAMPLE_CLK_2_3 : IN std_logic;
		SIGNAL_8_BIT_2_4 : IN std_logic_vector(7 downto 0);
		SIGNAL_8_BIT_SAMPLE_CLK_2_4 : IN std_logic;
		SIGNAL_16_BIT_2_1 : IN std_logic_vector(15 downto 0);
		SIGNAL_16_BIT_SAMPLE_CLK_2_1 : IN std_logic;
		SIGNAL_16_BIT_2_2 : IN std_logic_vector(15 downto 0);
		SIGNAL_16_BIT_SAMPLE_CLK_2_2 : IN std_logic;
		SIGNAL_16_BIT_2_3 : IN std_logic_vector(15 downto 0);
		SIGNAL_16_BIT_SAMPLE_CLK_2_3 : IN std_logic;
		SIGNAL_16_BIT_2_4 : IN std_logic_vector(15 downto 0);
		SIGNAL_16_BIT_SAMPLE_CLK_2_4 : IN std_logic;
		SIGNAL_1_BIT_3_1 : IN std_logic;
		SIGNAL_1_BIT_SAMPLE_CLK_3_1 : IN std_logic;
		SIGNAL_1_BIT_3_2 : IN std_logic;
		SIGNAL_1_BIT_SAMPLE_CLK_3_2 : IN std_logic;
		SIGNAL_1_BIT_3_3 : IN std_logic;
		SIGNAL_1_BIT_SAMPLE_CLK_3_3 : IN std_logic;
		SIGNAL_1_BIT_3_4 : IN std_logic;
		SIGNAL_1_BIT_SAMPLE_CLK_3_4 : IN std_logic;
		SIGNAL_2_BIT_3_1 : IN std_logic_vector(1 downto 0);
		SIGNAL_2_BIT_SAMPLE_CLK_3_1 : IN std_logic;
		SIGNAL_2_BIT_3_2 : IN std_logic_vector(1 downto 0);
		SIGNAL_2_BIT_SAMPLE_CLK_3_2 : IN std_logic;
		SIGNAL_2_BIT_3_3 : IN std_logic_vector(1 downto 0);
		SIGNAL_2_BIT_SAMPLE_CLK_3_3 : IN std_logic;
		SIGNAL_2_BIT_3_4 : IN std_logic_vector(1 downto 0);
		SIGNAL_2_BIT_SAMPLE_CLK_3_4 : IN std_logic;
		SIGNAL_4_BIT_3_1 : IN std_logic_vector(3 downto 0);
		SIGNAL_4_BIT_SAMPLE_CLK_3_1 : IN std_logic;
		SIGNAL_4_BIT_3_2 : IN std_logic_vector(3 downto 0);
		SIGNAL_4_BIT_SAMPLE_CLK_3_2 : IN std_logic;
		SIGNAL_4_BIT_3_3 : IN std_logic_vector(3 downto 0);
		SIGNAL_4_BIT_SAMPLE_CLK_3_3 : IN std_logic;
		SIGNAL_4_BIT_3_4 : IN std_logic_vector(3 downto 0);
		SIGNAL_4_BIT_SAMPLE_CLK_3_4 : IN std_logic;
		SIGNAL_8_BIT_3_1 : IN std_logic_vector(7 downto 0);
		SIGNAL_8_BIT_SAMPLE_CLK_3_1 : IN std_logic;
		SIGNAL_8_BIT_3_2 : IN std_logic_vector(7 downto 0);
		SIGNAL_8_BIT_SAMPLE_CLK_3_2 : IN std_logic;
		SIGNAL_8_BIT_3_3 : IN std_logic_vector(7 downto 0);
		SIGNAL_8_BIT_SAMPLE_CLK_3_3 : IN std_logic;
		SIGNAL_8_BIT_3_4 : IN std_logic_vector(7 downto 0);
		SIGNAL_8_BIT_SAMPLE_CLK_3_4 : IN std_logic;
		SIGNAL_16_BIT_3_1 : IN std_logic_vector(15 downto 0);
		SIGNAL_16_BIT_SAMPLE_CLK_3_1 : IN std_logic;
		SIGNAL_16_BIT_3_2 : IN std_logic_vector(15 downto 0);
		SIGNAL_16_BIT_SAMPLE_CLK_3_2 : IN std_logic;
		SIGNAL_16_BIT_3_3 : IN std_logic_vector(15 downto 0);
		SIGNAL_16_BIT_SAMPLE_CLK_3_3 : IN std_logic;
		SIGNAL_16_BIT_3_4 : IN std_logic_vector(15 downto 0);
		SIGNAL_16_BIT_SAMPLE_CLK_3_4 : IN std_logic;
		SIGNAL_1_BIT_4_1 : IN std_logic;
		SIGNAL_1_BIT_SAMPLE_CLK_4_1 : IN std_logic;
		SIGNAL_1_BIT_4_2 : IN std_logic;
		SIGNAL_1_BIT_SAMPLE_CLK_4_2 : IN std_logic;
		SIGNAL_1_BIT_4_3 : IN std_logic;
		SIGNAL_1_BIT_SAMPLE_CLK_4_3 : IN std_logic;
		SIGNAL_1_BIT_4_4 : IN std_logic;
		SIGNAL_1_BIT_SAMPLE_CLK_4_4 : IN std_logic;
		SIGNAL_2_BIT_4_1 : IN std_logic_vector(1 downto 0);
		SIGNAL_2_BIT_SAMPLE_CLK_4_1 : IN std_logic;
		SIGNAL_2_BIT_4_2 : IN std_logic_vector(1 downto 0);
		SIGNAL_2_BIT_SAMPLE_CLK_4_2 : IN std_logic;
		SIGNAL_2_BIT_4_3 : IN std_logic_vector(1 downto 0);
		SIGNAL_2_BIT_SAMPLE_CLK_4_3 : IN std_logic;
		SIGNAL_2_BIT_4_4 : IN std_logic_vector(1 downto 0);
		SIGNAL_2_BIT_SAMPLE_CLK_4_4 : IN std_logic;
		SIGNAL_4_BIT_4_1 : IN std_logic_vector(3 downto 0);
		SIGNAL_4_BIT_SAMPLE_CLK_4_1 : IN std_logic;
		SIGNAL_4_BIT_4_2 : IN std_logic_vector(3 downto 0);
		SIGNAL_4_BIT_SAMPLE_CLK_4_2 : IN std_logic;
		SIGNAL_4_BIT_4_3 : IN std_logic_vector(3 downto 0);
		SIGNAL_4_BIT_SAMPLE_CLK_4_3 : IN std_logic;
		SIGNAL_4_BIT_4_4 : IN std_logic_vector(3 downto 0);
		SIGNAL_4_BIT_SAMPLE_CLK_4_4 : IN std_logic;
		SIGNAL_8_BIT_4_1 : IN std_logic_vector(7 downto 0);
		SIGNAL_8_BIT_SAMPLE_CLK_4_1 : IN std_logic;
		SIGNAL_8_BIT_4_2 : IN std_logic_vector(7 downto 0);
		SIGNAL_8_BIT_SAMPLE_CLK_4_2 : IN std_logic;
		SIGNAL_8_BIT_4_3 : IN std_logic_vector(7 downto 0);
		SIGNAL_8_BIT_SAMPLE_CLK_4_3 : IN std_logic;
		SIGNAL_8_BIT_4_4 : IN std_logic_vector(7 downto 0);
		SIGNAL_8_BIT_SAMPLE_CLK_4_4 : IN std_logic;
		SIGNAL_16_BIT_4_1 : IN std_logic_vector(15 downto 0);
		SIGNAL_16_BIT_SAMPLE_CLK_4_1 : IN std_logic;
		SIGNAL_16_BIT_4_2 : IN std_logic_vector(15 downto 0);
		SIGNAL_16_BIT_SAMPLE_CLK_4_2 : IN std_logic;
		SIGNAL_16_BIT_4_3 : IN std_logic_vector(15 downto 0);
		SIGNAL_16_BIT_SAMPLE_CLK_4_3 : IN std_logic;
		SIGNAL_16_BIT_4_4 : IN std_logic_vector(15 downto 0);
		SIGNAL_16_BIT_SAMPLE_CLK_4_4 : IN std_logic;
		TRIGGER_1_BIT_1 : IN std_logic;
		TRIGGER_1_BIT_SAMPLE_CLK_1 : IN std_logic;
		TRIGGER_1_BIT_2 : IN std_logic;
		TRIGGER_1_BIT_SAMPLE_CLK_2 : IN std_logic;
		TRIGGER_1_BIT_3 : IN std_logic;
		TRIGGER_1_BIT_SAMPLE_CLK_3 : IN std_logic;
		TRIGGER_1_BIT_4 : IN std_logic;
		TRIGGER_1_BIT_SAMPLE_CLK_4 : IN std_logic;
		TRIGGER_2_BIT_1 : IN std_logic_vector(1 downto 0);
		TRIGGER_2_BIT_SAMPLE_CLK_1 : IN std_logic;
		TRIGGER_2_BIT_2 : IN std_logic_vector(1 downto 0);
		TRIGGER_2_BIT_SAMPLE_CLK_2 : IN std_logic;
		TRIGGER_2_BIT_3 : IN std_logic_vector(1 downto 0);
		TRIGGER_2_BIT_SAMPLE_CLK_3 : IN std_logic;
		TRIGGER_2_BIT_4 : IN std_logic_vector(1 downto 0);
		TRIGGER_2_BIT_SAMPLE_CLK_4 : IN std_logic;
		TRIGGER_4_BIT_1 : IN std_logic_vector(3 downto 0);
		TRIGGER_4_BIT_SAMPLE_CLK_1 : IN std_logic;
		TRIGGER_4_BIT_2 : IN std_logic_vector(3 downto 0);
		TRIGGER_4_BIT_SAMPLE_CLK_2 : IN std_logic;
		TRIGGER_4_BIT_3 : IN std_logic_vector(3 downto 0);
		TRIGGER_4_BIT_SAMPLE_CLK_3 : IN std_logic;
		TRIGGER_4_BIT_4 : IN std_logic_vector(3 downto 0);
		TRIGGER_4_BIT_SAMPLE_CLK_4 : IN std_logic;
		TRIGGER_8_BIT_1 : IN std_logic_vector(7 downto 0);
		TRIGGER_8_BIT_SAMPLE_CLK_1 : IN std_logic;
		TRIGGER_8_BIT_2 : IN std_logic_vector(7 downto 0);
		TRIGGER_8_BIT_SAMPLE_CLK_2 : IN std_logic;
		TRIGGER_8_BIT_3 : IN std_logic_vector(7 downto 0);
		TRIGGER_8_BIT_SAMPLE_CLK_3 : IN std_logic;
		TRIGGER_8_BIT_4 : IN std_logic_vector(7 downto 0);
		TRIGGER_8_BIT_SAMPLE_CLK_4 : IN std_logic;
		TRIGGER_16_BIT_1 : IN std_logic_vector(15 downto 0);
		TRIGGER_16_BIT_SAMPLE_CLK_1 : IN std_logic;
		TRIGGER_16_BIT_2 : IN std_logic_vector(15 downto 0);
		TRIGGER_16_BIT_SAMPLE_CLK_2 : IN std_logic;
		TRIGGER_16_BIT_3 : IN std_logic_vector(15 downto 0);
		TRIGGER_16_BIT_SAMPLE_CLK_3 : IN std_logic;
		TRIGGER_16_BIT_4 : IN std_logic_vector(15 downto 0);
		TRIGGER_16_BIT_SAMPLE_CLK_4 : IN std_logic;
		TRIGGER_REARM_TOGGLE : IN std_logic;
		FORCE_TRIGGER_TOGGLE : IN std_logic;
		SREG250_READ_TOGGLE : IN std_logic;
		START_CAPTURE_TOGGLE : IN std_logic;          
		REG250 : OUT std_logic_vector(7 downto 0);
		REG251 : OUT std_logic_vector(7 downto 0);
		TP : OUT std_logic_vector(10 downto 1)
		);
	END COMPONENT;

--------------------------------------------------------
--     SIGNALS
--------------------------------------------------------
-- Suffix _D indicates a one CLK delayed version of the net with the same name
-- Suffix _X indicates an extended precision version of the net with the same name
-- Suffix _N indicates an inverted version of the net with the same name

--// CLOCKS, RESETS
signal CLK_REFg: std_logic;
signal CLK_P: std_logic;	-- main processing clock
signal CLK_IO: std_logic;
signal CLK_IO_D: std_logic;

signal PLL_LOCKED: std_logic;
signal ASYNC_RESET: std_logic;
signal ASYNC_RESET_D: std_logic;
signal ASYNC_RESET_D2: std_logic;
signal SYNC_RESET: std_logic;
signal RESET_COUNTER: std_logic_vector(23 downto 0) := x"000000";	-- initialize for simulations
	-- spans 2^23/20MHz = 420ms
signal ADC_CLK1: std_logic;

--// DETECT SAMPLING FREQUENCY ERROR  -----------------------
signal CLK_REF_CNTR: std_logic_vector(24 downto 0) := (others => '0');
signal TOGGLE_1s: std_logic := '0';
signal TOGGLE_1s_D: std_logic := '0';
signal TOGGLE_1s_D2: std_logic := '0';
signal CLK_P_CNTR: std_logic_vector(26 downto 0) := (others => '0');
signal CLK_P_FREQ: std_logic_vector(26 downto 0) := (others => '0');

	
--// RECLOCK INPUTS ---------------------
signal ADC_DATA_IN_D: std_logic_vector(13 downto 0);

--// PEAK DETECT -------------------------
signal INPUT_PEAK_AMPLITUDE_MSB: std_logic_vector(7 downto 0);	-- measure the peak amplitude in a 256 sample window.
signal PEAK_DETECT_CNTR: std_logic_vector(7 downto 0);	-- modulo 256 counter to detect the peak amplitude of input samples
signal ADC_DATA_IN_ABS: std_logic_vector(7 downto 0);
signal ADC_DATA_IN_ABS_MAX: std_logic_vector(7 downto 0);

--// FREQUENCY TRANSLATION ---------------
signal COUNTER4: std_logic_vector(1 downto 0) := "00";
signal DATA2_I: std_logic_vector(13 downto 0);
signal DATA2_Q: std_logic_vector(13 downto 0);

-- VARIABLE DECIMATION  PART I : CIC DECIMATION  -----------------------
signal CIC_RESET: std_logic := '0';
signal CIC_R: std_logic_vector(15 downto 0);
signal DATA3_I: std_logic_vector(11 downto 0);
signal DATA3_Q: std_logic_vector(11 downto 0);
signal SAMPLE3_CLK: std_logic;
signal DATA3_I_D: std_logic_vector(11 downto 0);
signal DATA3_Q_D: std_logic_vector(11 downto 0);
signal SAMPLE3_CLK_D: std_logic;
signal CIC_N: integer range 1 to 8;
		-- decimation filter: number of stages
signal CIC_M: integer range 1 to 2;
		-- decimation filter: differential delay
signal CIC_TRUNCATE_OBITS: integer range 0 to 63;
		-- decimation filter: number of least significant bits to be truncated at the output. 
		-- Must be set to N*log2(RM), which is the bit growth during the CIC
		-- calculation due to the gain G = (RM)^N.
		-- As this CIC module is designed to be flexible, it is generally more efficient
		-- to compute this field by the application (i.e. outside this module) 
		-- since the parameters N,R,M are generally known and fixed at the time of 
		-- synthesis.
signal CIC_R_BIN_DEC: std_logic_vector(15 downto 0);
		-- intermediate computation: R-1
signal R: integer range 1 to 16384;

-- FREQUENCY TRANSLATION ----------------------------------------------------------------
signal NCO_FREQ: std_logic_vector(31 downto 0);
signal NCO_FREQB: std_logic_vector(31 downto 0);
signal PHASE_ROTATION: std_logic_vector(31 downto 0);
signal DATA4_I: std_logic_vector(11 downto 0);
signal DATA4_Q: std_logic_vector(11 downto 0);
signal SAMPLE4_CLK: std_logic;

-- VARIABLE BANDWIDTH FILTER  PART III: CONCATENATED HALF-BAND FILTERS --------------------------
signal DATA4_I_X: std_logic_vector(13 downto 0);
signal DATA4_Q_X: std_logic_vector(13 downto 0);
signal DATA5_I: std_logic_vector(13 downto 0);
signal DATA5_Q: std_logic_vector(13 downto 0);
signal SAMPLE5_CLK: std_logic;
signal DATA5_I_D: std_logic_vector(13 downto 0);
signal DATA5_Q_D: std_logic_vector(13 downto 0);
signal SAMPLE5_CLK_D: std_logic;
signal SAMPLE5B_CLK: std_logic;
signal S5TOGGLE: std_logic := '0';
signal DATA6_I: std_logic_vector(13 downto 0);
signal DATA6_Q: std_logic_vector(13 downto 0);
signal DATA6B_I: std_logic_vector(11 downto 0);
signal DATA6B_Q: std_logic_vector(11 downto 0);
signal SAMPLE6_CLK: std_logic;
signal SAMPLE6B_CLK: std_logic;
signal DATA6_I_X: std_logic_vector(17 downto 0);
signal DATA6_Q_X: std_logic_vector(17 downto 0);

--// GAIN CONTROL ----------------------------------------------
signal DATA7_I: std_logic_vector(35 downto 0);
signal DATA7_Q: std_logic_vector(35 downto 0);
signal DATA7_I_D: std_logic_vector(35 downto 0);
signal DATA7_Q_D: std_logic_vector(35 downto 0);
signal SAMPLE7_CLK_D: std_logic;
signal DATA8_I: std_logic_vector(11 downto 0);
signal DATA8_Q: std_logic_vector(11 downto 0);
signal SAMPLE8_CLK: std_logic;
signal SIGNAL_SCALING: std_logic_vector(17 downto 0);

--// OPTIONAL RESAMPLING AT 40 MS/s ----------------------------
signal DATA9_I: std_logic_vector(11 downto 0);
signal DATA9_Q: std_logic_vector(11 downto 0);
signal DATA9_I_D: std_logic_vector(11 downto 0);
signal DATA9_Q_D: std_logic_vector(11 downto 0);

--// OUTPUT ----------------------------------------------------
signal CLKOUT: std_logic;
signal DATAOUT_I: std_logic_vector(11 downto 0);
signal DATAOUT_Q: std_logic_vector(11 downto 0);

--// ComScope traces
signal S1_1: std_logic_vector(7 downto 0);	
signal S1_1_CLK: std_logic;
signal S1_1B_CLK: std_logic;
signal S1_2: std_logic_vector(7 downto 0);	
signal S1_2_CLK: std_logic;
signal S1_2B_CLK: std_logic;
signal S1_3: std_logic_vector(7 downto 0);	
signal S1_3_CLK: std_logic;
signal S1_3B_CLK: std_logic;
signal S1_4: std_logic_vector(7 downto 0);	
signal S1_4_CLK: std_logic;
signal S1_4B_CLK: std_logic;
signal S2_1: std_logic_vector(7 downto 0);	
signal S2_1_CLK: std_logic;
signal S2_1B_CLK: std_logic;
signal S2_2: std_logic_vector(7 downto 0);	
signal S2_2_CLK: std_logic;
signal S2_2B_CLK: std_logic;
signal S2_3: std_logic_vector(7 downto 0);	
signal S2_3_CLK: std_logic;
signal S2_3B_CLK: std_logic;
signal S2_4: std_logic_vector(7 downto 0);	
signal S2_4_CLK: std_logic;
signal S2_4B_CLK: std_logic;
signal S3_1: std_logic_vector(7 downto 0);	
signal S3_1_CLK: std_logic;
signal S3_1B_CLK: std_logic;
signal S3_2: std_logic_vector(7 downto 0);	
signal S3_2_CLK: std_logic;
signal S3_2B_CLK: std_logic;
signal S3_3: std_logic_vector(7 downto 0);	
signal S3_3_CLK: std_logic;
signal S3_3B_CLK: std_logic;
signal S3_4: std_logic_vector(7 downto 0);	
signal S3_4_CLK: std_logic;
signal S3_4B_CLK: std_logic;

signal T1: std_logic;
signal T1_CLK: std_logic;
signal T2: std_logic;
signal T2_CLK: std_logic;

signal TRIGGER_REARM_TOGGLE: std_logic:= '0';
signal FORCE_TRIGGER_TOGGLE: std_logic:= '0';
signal START_CAPTURE_TOGGLE: std_logic:= '0';
signal SREG250_READ_TOGGLE: std_logic;

--// ARM co-processor interface
signal UC_WEBG: std_logic;  -- WE# through global buffer
signal UC_ADDRESS: std_logic_vector(7 downto 0);
signal UC_AD_LOCAL: std_logic_vector(7 downto 0);
signal UC_RDN_D: std_logic;
signal UC_RDN_D2: std_logic;
signal CONFIG_CHANGE_TOGGLE: std_logic;
signal CONFIG_CHANGE_TOGGLE_D: std_logic;
signal CONFIG_CHANGE_TOGGLE_D2: std_logic;
signal CONFIG_CHANGE_PULSE: std_logic;
signal CONFIG_CHANGE_PULSE_D: std_logic;
signal CONFIG_CHANGE_PULSE_D2: std_logic;
signal SIGNAL_SCALING_CHANGE_TOGGLE: std_logic;
signal SIGNAL_SCALING_CHANGE_TOGGLE_D: std_logic;
signal SIGNAL_SCALING_CHANGE_TOGGLE_D2: std_logic;
signal SIGNAL_SCALING_CHANGE_PULSE: std_logic;


--// Control Registers
signal REG100: std_logic_vector(7 downto 0);
signal REG101: std_logic_vector(7 downto 0);
signal REG102: std_logic_vector(7 downto 0);
signal REG103: std_logic_vector(7 downto 0);
signal REG70: std_logic_vector(7 downto 0);
signal REG71: std_logic_vector(7 downto 0);
signal REG6: std_logic_vector(7 downto 0);
signal REG107: std_logic_vector(7 downto 0);
signal REG108: std_logic_vector(7 downto 0);

--// Status Registers
signal SREG0: std_logic_vector(7 downto 0);
signal SREG1: std_logic_vector(7 downto 0);
signal SREG2: std_logic_vector(7 downto 0);
signal SREG3: std_logic_vector(7 downto 0);
signal SREG4: std_logic_vector(7 downto 0);
signal SREG5: std_logic_vector(7 downto 0);
signal SREG6: std_logic_vector(7 downto 0);
signal SREG7: std_logic_vector(7 downto 0);
signal SREG8: std_logic_vector(7 downto 0);
signal SREG9: std_logic_vector(7 downto 0);
signal SREG10: std_logic_vector(7 downto 0);
signal SREG254_READ_TOGGLE: std_logic;			

--// ComScope control registers
signal REG237: std_logic_vector(7 downto 0):= "00000000";
signal REG238: std_logic_vector(7 downto 0):= "00000000";
signal REG239: std_logic_vector(7 downto 0):= "00000000";
signal REG240: std_logic_vector(7 downto 0):= "00001101";	-- Trace 1 signal selection
signal REG241: std_logic_vector(7 downto 0):= "00000000";	-- Trigger position, decimation
signal REG242: std_logic_vector(7 downto 0):= "00001110";	-- Trace 2 signal selection
signal REG243: std_logic_vector(7 downto 0):= "00000000";	-- Trigger position, decimation
signal REG244: std_logic_vector(7 downto 0):= "00000000";	-- Trace 3 signal selection
signal REG245: std_logic_vector(7 downto 0):= "00000000";	-- Trigger position, decimation
signal REG246: std_logic_vector(7 downto 0):= "00000000";	-- Trace 4 signal selection
signal REG247: std_logic_vector(7 downto 0):= "00000000";	-- Trigger position, decimation
signal REG248: std_logic_vector(7 downto 0):= "10001101";	-- Trigger edge, trigger signal
signal REG249: std_logic_vector(7 downto 0):= "10111111";	-- Trigger threshold
signal REG250: std_logic_vector(7 downto 0):= "00000000";
signal REG251: std_logic_vector(7 downto 0):= "00000000";


--// FPGA option and revision (ASCII format)
-- These two ASCII characters appear in the Comblock Control Center.
-- Very helpful in identifying which firmware version is currently running
-- in the FPGA. 
-- '0' = 48
-- 'A' = 65
constant OPTION: std_logic_vector(7 downto 0) := x"20";	-- ' '
constant REVISION: std_logic_vector(7 downto 0) := x"34";	-- '4'
--------------------------------------------------------
--      IMPLEMENTATION
--------------------------------------------------------
begin
--// CLOCKS, RESET ---------------------
-- Generate miscellaneous global clocks and resets from the 
-- reference 20 MHz clock  
-- FOR MOST APPLICATIONS, KEEP THIS SECTION UNCHANGED.
-- Most Common changes: change the various system clock frequencies within CLKGEN_PLL 

RESET_001: process(CLK_REFg)
begin
	if rising_edge(CLK_REFg) then
		-- PLL need time to lock. The reset of the application is still in reset
		if(RESET_COUNTER(17) = '0') then
			-- wait a bit before asserting the PLL lock
			RESET_COUNTER <= RESET_COUNTER + 1;
		end if;
	end if;
end process;

ASYNC_RESET <= not(RESET_COUNTER(17));	-- 6.5ms

-- generate a reset pulse synchronous with CLK_P (needed for CIC filter)
RESET_002: process(CLK_P, ASYNC_RESET)
begin
	if rising_edge(CLK_P) then
		ASYNC_RESET_D <= ASYNC_RESET;
		ASYNC_RESET_D2 <= ASYNC_RESET_D;
		if(ASYNC_RESET_D = '0') and (ASYNC_RESET_D2 = '1') then
			SYNC_RESET <= '1';
		else
			SYNC_RESET <= '0';
		end if;
	end if;
end process;

-- select ADC sampling clock source
ADC_CLK1 <= ADC_CLK when (REG6(1) = '0') else EXT_ADC_CLK;
-- forward ADC sampling clock to ADC when appropriate
ADC_CLK <= 'Z' when (REG6(1) = '0') else EXT_ADC_CLK;

-- Global buffers
BUFG_001: BUFG  port map (I=> ADC_CLK1, O=> CLK_P);		
	-- internal processing clock = ADC sampling clock, irrespective of its origin (100 MHz oscillator, from user through 40-pin connector...)

BUFG_002: BUFG  port map (I=> UC_WEB_IN, O=> UC_WEBG);	
	-- ARM WE#


Inst_CLKGEN_PLL: CLKGEN_PLL PORT MAP(
	CLKREF_IN => CLK_REF,
	CLKOUT0G => CLK_REFG,		-- 20 MHz reference clock, global
	CLKOUT2G => CLK_IO,			-- 40 MHz I/O clock
	LOCKED_PLL => PLL_LOCKED
);


--// DETECT SAMPLING FREQUENCY ERROR  -----------------------
-- The received signal undergoes three frequency translations: 
-- 1. The first frequency translation uses a RF synthesizer locked onto an internal 
--    or external 10/20 MHz frequency reference.
--    Internal stability is typically [-10 to +20 ppm] over temperature/aging/etc.
--    External stability depends on the frequency source (TCXO, OCXO, etc).
--	2. The two other frequency translations use the 100 MHz sampling clock as frequency reference.

-- 	The purpose of this circuit is to assess (and possibly compensate for)
-- 	the ADC_CLK (=CLK_P) sampling clock frequency error using CLK_REF as frequency reference.

-- counter with 1 second period. range 0 - 19999999
CLK_REF_COUNTER_GEN: process(CLK_REFg)
begin
	if(ASYNC_RESET = '1') then
		CLK_REF_CNTR <= (others => '0');
		TOGGLE_1s <= '0';
	elsif rising_edge(CLK_REFg) then
		if(CLK_REF_CNTR = 19999999) then	
			CLK_REF_CNTR <= (others => '0');
			TOGGLE_1s <= not TOGGLE_1s;
		else
			CLK_REF_CNTR <= CLK_REF_CNTR + 1;
		end if;
	end if;
end process;

CLK_P_COUNTER_GEN: process(CLK_P)
begin
	if(ASYNC_RESET = '1') then
		CLK_P_CNTR <= (others => '0');
		TOGGLE_1s_D <= '0';
		TOGGLE_1s_D2 <= '0';
	elsif rising_edge(CLK_P) then
		-- reclock 1s toggle with CLK_P
		TOGGLE_1s_D <= TOGGLE_1s;
		TOGGLE_1s_D2 <= TOGGLE_1s_D;
		
		if(TOGGLE_1s_D /= TOGGLE_1s_D2) then
			-- 1s tick. reset counter.
			CLK_P_CNTR <= (others => '0');
			-- save last frequency measurement
			CLK_P_FREQ <= CLK_P_CNTR + 1;
		else
			CLK_P_CNTR <= CLK_P_CNTR + 1;
		end if;
	end if;
end process;

-- report frequency error
SREG7 <= CLK_P_FREQ(7 downto 0);
-- other bytes are latched in upon reading status register SREG7. (search for SREG8/9/10)




--// RECLOCK INPUTS ---------------------
-- Immediately reclock inputs for best timing.
-- Already in 2's complement format
RECLOCK_IN_001: process(CLK_P)
begin
	if rising_edge(CLK_P) then
		ADC_DATA_IN_D(12 downto 0) <= ADC_DATA_IN(12 downto 0);
		ADC_DATA_IN_D(13) <= ADC_DATA_IN(13);		
	end if;
end process;

--// PEAK DETECT -------------------------
-- Detect the input sample (from A/D) peak amplitude within a 256-sample window.
-- Then report to the ARM processor which acts as gain controller (when enabled)
PEAK_DETECT_001: process(CLK_P)
begin
	if rising_edge(CLK_P) then
		PEAK_DETECT_CNTR <= PEAK_DETECT_CNTR + 1;	-- modulo-256 counter
		
		-- abs(input sample), 8 MSbs only. approximation (1 off for negative numbers)
		if(ADC_DATA_IN_D(13) = '0') then
			ADC_DATA_IN_ABS <= ADC_DATA_IN_D(12 downto 5);
		else
			ADC_DATA_IN_ABS <= not ADC_DATA_IN_D(12 downto 5);
		end if;
			
		if(PEAK_DETECT_CNTR = 0) then
			-- first sample in new window
			ADC_DATA_IN_ABS_MAX <= ADC_DATA_IN_ABS;	-- first sample is always max
			INPUT_PEAK_AMPLITUDE_MSB <= ADC_DATA_IN_ABS_MAX;	-- max in previous window
		elsif (ADC_DATA_IN_ABS > ADC_DATA_IN_ABS_MAX) then
			-- new max
			ADC_DATA_IN_ABS_MAX <= ADC_DATA_IN_ABS;
		end if;
	end if;
end process;

SREG1 <= INPUT_PEAK_AMPLITUDE_MSB;
	-- report the input samples peak amplitude to the ARM processor for gain control purposes.


--// FREQUENCY TRANSLATION ---------------
---- coarse frequency translation (sometimes, this is just sufficient and of course uses fewer resources in the FPGA)
---- first move the center frequency from 25 to 0 MHz after IF undersampling
---- De-rotate by -25 MHz
--FREQ_TRANSLATION_001: process(CLK_P) 
--begin
--	if rising_edge(CLK_P) then
--		COUNTER4 <= COUNTER4 + 1;
--		case COUNTER4 is
--			when "00" => 
--				DATA2_I <= ADC_DATA_IN_D;
--				DATA2_Q <= (others => '0');
--			when "01" => 
--				DATA2_I <= (others => '0');
--				DATA2_Q <= not(ADC_DATA_IN_D);
--			when "10" => 
--				DATA2_I <= not(ADC_DATA_IN_D);
--				DATA2_Q <= (others => '0');
--			when others => 
--				DATA2_I <= (others => '0');
--				DATA2_Q <= ADC_DATA_IN_D;
--		end case;
--	end if;
--end process;

-- fine frequency translation 
Inst_DIGITAL_DC2: DIGITAL_DC2 PORT MAP(
	CLK => CLK_P,
	DATA_I_IN => ADC_DATA_IN_D(13 downto 2),
	DATA_Q_IN => x"000",
	SAMPLE_CLK_IN => '1',
	PHASE_ROTATION => PHASE_ROTATION(31 downto 20),
	DATA_I_OUT => DATA2_I(13 downto 2),
	DATA_Q_OUT => DATA2_Q(13 downto 2),
	SAMPLE_CLK_OUT => open,
	SATURATION => open
);

-- frequency translation is + value in registers 100-103 -fclk/4
NCO_FREQB(29 downto 0) <= NCO_FREQ(29 downto 0);
NCO_FREQB(31 downto 30) <= NCO_FREQ(31 downto 30) + "11";


Inst_NCO32: NCO32 PORT MAP(
	CLK => CLK_P,
	SYNC_RESET => SYNC_RESET,
	NOMINAL_FREQUENCY => NCO_FREQB,
	PHASE_OFFSET => x"00000000",
	PHASE_OFFSET_SAMPLE_CLK => '0',
	PHASE => PHASE_ROTATION
);


--// VARIABLE BANDWIDTH FILTER  PART I : CIC FILTER  -----------------------
-- Use CIC filters to reject far-out, half-band filter to reject close to the target bandwidth = best of breeds.
-- Two filters, one each for I and Q.
-- The decimation rate R must be selected such that fclk/R is strictly greater than
-- 4* symbol rate. 
CIC_N <= 2;
CIC_M <= 1;   
CIC_R_BIN_DEC <= CIC_R -1;

-- compute the number of bits to truncate: 
-- reclock for best timing. Update only at reset.
CIC_TRUNCATE_OBITS_GEN: process(SYNC_RESET, CLK_P, CIC_R_BIN_DEC)
begin
	if rising_edge(CLK_P) then
		if(CONFIG_CHANGE_PULSE_D = '1') then
			if(CIC_R_BIN_DEC(15) = '1') then
				CIC_TRUNCATE_OBITS <= 32;
			elsif(CIC_R_BIN_DEC(14) = '1') then
				CIC_TRUNCATE_OBITS <= 30;
			elsif(CIC_R_BIN_DEC(13) = '1') then
				CIC_TRUNCATE_OBITS <= 28;
			elsif(CIC_R_BIN_DEC(12) = '1') then
				CIC_TRUNCATE_OBITS <= 26;
			elsif(CIC_R_BIN_DEC(11) = '1') then
				CIC_TRUNCATE_OBITS <= 24;
			elsif(CIC_R_BIN_DEC(10) = '1') then
				CIC_TRUNCATE_OBITS <= 22;
			elsif(CIC_R_BIN_DEC(9) = '1') then
				CIC_TRUNCATE_OBITS <= 20;
			elsif(CIC_R_BIN_DEC(8) = '1') then
				CIC_TRUNCATE_OBITS <= 18;
			elsif(CIC_R_BIN_DEC(7) = '1') then
				CIC_TRUNCATE_OBITS <= 16;
			elsif(CIC_R_BIN_DEC(6) = '1') then
				CIC_TRUNCATE_OBITS <= 14;
			elsif(CIC_R_BIN_DEC(5) = '1') then
				CIC_TRUNCATE_OBITS <= 12;
			elsif(CIC_R_BIN_DEC(4) = '1') then
				CIC_TRUNCATE_OBITS <= 10;
			elsif(CIC_R_BIN_DEC(3) = '1') then
				CIC_TRUNCATE_OBITS <= 8;
			elsif(CIC_R_BIN_DEC(2) = '1') then
				CIC_TRUNCATE_OBITS <= 6;
			elsif(CIC_R_BIN_DEC(1) = '1') then
				CIC_TRUNCATE_OBITS <= 4;
			elsif(CIC_R_BIN_DEC(0) = '1') then
				CIC_TRUNCATE_OBITS <= 2;
			else
				CIC_TRUNCATE_OBITS <= 0;
			end if;
		end if;
	end if;
end process;

R <= conv_integer(CIC_R);

CIC_RESET <= SYNC_RESET or CONFIG_CHANGE_PULSE_D2;
	

CIC_FILTER_001: CIC port map(
	CLK => CLK_P,
	SYNC_RESET => CIC_RESET, -- CIC must be reset upon changing decimation ratio, otherwise saturation will occur.
	DATA_IN => DATA2_I(13 downto 2),
	SAMPLE_CLK_IN => '1',	
	N => CIC_N,
	R => R,
	M => CIC_M,
	TRUNCATE_OBITS => CIC_TRUNCATE_OBITS,
	DATA_OUT => DATA3_I,
	SAMPLE_CLK_OUT => SAMPLE3_CLK
	);

CIC_FILTER_002: CIC port map(
	CLK => CLK_P,
	SYNC_RESET => CIC_RESET,
	DATA_IN => DATA2_Q(13 downto 2),
	SAMPLE_CLK_IN => '1',	
	N => CIC_N,
	R => R,
	M => CIC_M,
	TRUNCATE_OBITS => CIC_TRUNCATE_OBITS,
	DATA_OUT => DATA3_Q,
	SAMPLE_CLK_OUT => open
	);

-- bypass CIC filter
BYPASS_CIC: process(CLK_P)
begin
	if rising_edge(CLK_P) then
		if(REG108(7 downto 6) /= "11") then
			-- bypass CIC filter
			DATA3_I_D <= DATA2_I(13 downto 2);
			DATA3_Q_D <= DATA2_Q(13 downto 2);
			SAMPLE3_CLK_D <= '1';
		else
			-- no bypass
			DATA3_I_D <= DATA3_I;
			DATA3_Q_D <= DATA3_Q;
			SAMPLE3_CLK_D <= SAMPLE3_CLK;
		end if;
	end if;
end process;


-- VARIABLE BANDWIDTH FILTER  PART II: CONCATENATED HALF-BAND FILTERS --------------------------
DATA4_I_X <= DATA3_I_D & "00";
DATA4_Q_X <= DATA3_Q_D & "00";
SAMPLE4_CLK <= SAMPLE3_CLK_D;

FIRHALFBAND3_I1: FIRHALFBAND3 PORT MAP(
	CLK => CLK_P,
	SAMPLE_CLK_IN => SAMPLE4_CLK,
	DATA_IN => DATA4_I_X,
	SAMPLE_CLK_OUT => SAMPLE5_CLK,
	DATA_OUT => DATA5_I
);

FIRHALFBAND3_Q1: FIRHALFBAND3 PORT MAP(
	CLK => CLK_P,
	SAMPLE_CLK_IN => SAMPLE4_CLK,
	DATA_IN => DATA4_Q_X,
	SAMPLE_CLK_OUT => open,
	DATA_OUT => DATA5_Q
);

-- decimate by 2 prior to the second stage of half-band filters
S5TOGGLE_GEN: process(CLK_P)
begin
	if rising_edge(CLK_P) then
		if(SAMPLE5_CLK = '1') then
			S5TOGGLE <= not S5TOGGLE;
		end if;
	end if;
end process;
			
SAMPLE5B_CLK <= SAMPLE5_CLK and S5TOGGLE;

-- bypass 1st HBF filter
BYPASS_1STHBF: process(CLK_P)
begin
	if rising_edge(CLK_P) then
		if(REG108(7) = '0') then
			-- bypass CIC filter
			DATA5_I_D <= DATA2_I;
			DATA5_Q_D <= DATA2_Q;
			SAMPLE5_CLK_D <= '1';
		else
			-- no bypass
			DATA5_I_D <= DATA5_I;
			DATA5_Q_D <= DATA5_Q;
			SAMPLE5_CLK_D <= SAMPLE5B_CLK;
		end if;
	end if;
end process;


FIRHALFBAND3_I2: FIRHALFBAND3 PORT MAP(
	CLK => CLK_P,
	SAMPLE_CLK_IN => SAMPLE5_CLK_D,
	DATA_IN => DATA5_I_D,
	SAMPLE_CLK_OUT => SAMPLE6_CLK,
	DATA_OUT => DATA6_I
);

FIRHALFBAND3_Q2: FIRHALFBAND3 PORT MAP(
	CLK => CLK_P,
	SAMPLE_CLK_IN => SAMPLE5_CLK_D,
	DATA_IN => DATA5_Q_D,
	SAMPLE_CLK_OUT => open,
	DATA_OUT => DATA6_Q
);

--// INTERPOLATE to smooth the waveform before output and/or 40 MHz resampling
CIC_INTERPOL3_001: CIC_INTERPOL3 PORT MAP(
	CLK => CLK_P,
	SYNC_RESET => CIC_RESET,
	DATA_IN => DATA6_I(13 downto 2),
	R => R,
	DATA_OUT => DATA6B_I,
	SATURATION => open
);

CIC_INTERPOL3_002: CIC_INTERPOL3 PORT MAP(
	CLK => CLK_P,
	SYNC_RESET => CIC_RESET,
	DATA_IN => DATA6_Q(13 downto 2),
	R => R,
	DATA_OUT => DATA6B_Q,
	SATURATION => open
);
SAMPLE6B_CLK <= '1';

--// GAIN CONTROL ----------------------------------------------
-- amplitude scaling
DATA6_I_X <= DATA6B_I & "000000";	-- extend precision to match 18x18 multiplier's
DATA6_Q_X <= DATA6B_Q & "000000";

AMPLITUDE_CONTROL_001: MULT18X18SIGNED port map(
	A => DATA6_I_X,
	B => SIGNAL_SCALING,
	P => DATA7_I
	);
	 
AMPLITUDE_CONTROL_002: MULT18X18SIGNED port map(
	A => DATA6_Q_X,
	B => SIGNAL_SCALING,
	P => DATA7_Q
	);

-- reclock multiplier output for timing purposes.
RECLOCK_004: process(CLK_P)
begin 
	if rising_edge(CLK_P) then
		SAMPLE7_CLK_D <= SAMPLE6B_CLK;	 
		if(SAMPLE6B_CLK = '1') then
			DATA7_I_D <= DATA7_I;
			DATA7_Q_D <= DATA7_Q;
		end if;
	end if;
end process;

-- Drop the least significant bits
-- Clamp to prevent saturation when SIGNAL_SCALING gain is too high.
-- IMPORTANT: there must be enough digital signal gain to reach saturation 
-- (otherwise the follow-on module may not be able to properly detect saturation and adjust the gain downwards)
CLAMP_001: process(CLK_P)
begin 
	if rising_edge(CLK_P) then
		SAMPLE8_CLK <= SAMPLE7_CLK_D;
		
		if(SAMPLE7_CLK_D = '1') then
			if(DATA7_I_D(35) = '0') and (DATA7_I_D(34 downto 30) /= "00000") then
				-- clamp to most positive
				DATA8_I <= "011111111111";
			elsif(DATA7_I_D(35) = '1') and (DATA7_I_D(34 downto 30) /= "11111") then
				-- clamp to most negative
				DATA8_I <= "100000000001";
			else
				DATA8_I <= DATA7_I_D(30 downto 19);
			end if;
	
			if(DATA7_Q_D(35) = '0') and (DATA7_Q_D(34 downto 30) /= "00000") then
				-- clamp to most positive
				DATA8_Q <= "011111111111";
			elsif(DATA7_Q_D(35) = '1') and (DATA7_Q_D(34 downto 30) /= "11111") then
				-- clamp to most negative
				DATA8_Q <= "100000000001";
			else
				DATA8_Q <= DATA7_Q_D(30 downto 19);
			end if;
		end if;
	end if;
end process;

-- report output sample amplitude (MSbs) for baseband AGC. Format: unsigned.
SREG2 <= (not DATA8_I(11)) & DATA8_I(10 downto 4);	
SREG3 <= (not DATA8_Q(11)) & DATA8_Q(10 downto 4);	


--// OPTIONAL RESAMPLING AT 40 MS/s ----------------------------
-- output sampling clock depends on the output configuration
-- 40 MHz (CLK_IO) or 100 MHz (CLK_P) 
-- Be aware that the 40 MHz clock may not be locked onto the same reference as the 100 MHz clock.

-- reclock when CLK_IO = '1' 
RECLOCK_OUTPUT_001: process(CLK_P, CLK_IO)
begin
	if rising_edge(CLK_P) then
		CLK_IO_D <= CLK_IO;
		if(CLK_IO_D = '1') then
			-- direct to A side
			DATA9_I <= DATA8_I;
			DATA9_Q <= DATA8_Q;
		end if;
	end if;
end process;

-- reclock input samples with IO clock when input samples are stable
-- stable in the A or B buffer.
RECLOCK_OUTPUT_004: process(CLK_IO)
begin
	if rising_edge(CLK_IO) then
		DATA9_I_D <= DATA9_I;
		DATA9_Q_D <= DATA9_Q;
	end if;
end process;



--// OUTPUT ----------------------------------------------------
-- Output mux 40 MHz samples or 100 MHz samples? Depends on REG6(3)
CLKOUT <= (CLK_P) when (REG6(3) = '1') else (CLK_IO);
DATAOUT_I <= DATA8_I when (REG6(3) = '1') else DATA9_I_D;
DATAOUT_Q <= DATA8_Q when (REG6(3) = '1') else DATA9_Q_D;

--ODDR2_001 : ODDR2
--generic map(
--	DDR_ALIGNMENT => "NONE", -- Sets output alignment to "NONE", "C0", "C1" 
--	INIT => '0', -- Sets initial state of the Q output to '0' or '1'
--	SRTYPE => "SYNC") -- Specifies "SYNC" or "ASYNC" set/reset
--port map (
--	Q => CLK_OUT, -- 1-bit output data
--	C0 => CLKOUT, -- 1-bit clock input
--	C1 => not CLKOUT, -- 1-bit clock input
--	CE => '1',  -- 1-bit clock enable input
--	D0 => '0',   -- 1-bit data input (associated with C0)
--	D1 => '1',   -- 1-bit data input (associated with C1)
--	R => '0',    -- 1-bit reset input
--	S => '1'     -- 1-bit set input
--);

CLK_OUT <= not CLKOUT;	-- invert for better timing
OUTPUT(0) <= '1';
OUTPUT(1) <= not DATAOUT_I(11); -- convert to unsigned output
OUTPUT(2) <= DATAOUT_I(10);
OUTPUT(3) <= DATAOUT_I(9);
OUTPUT(4) <= DATAOUT_I(8);
OUTPUT(5) <= DATAOUT_I(7);
OUTPUT(6) <= DATAOUT_I(6);
OUTPUT(7) <= DATAOUT_I(5);
OUTPUT(8) <= DATAOUT_I(4);
OUTPUT(9) <= DATAOUT_I(3);
OUTPUT(10) <= DATAOUT_I(2);
OUTPUT(11) <= not DATAOUT_Q(11); -- convert to unsigned output
OUTPUT(12) <= DATAOUT_Q(10);
OUTPUT(13) <= DATAOUT_Q(9);
OUTPUT(14) <= DATAOUT_Q(8);
OUTPUT(15) <= DATAOUT_Q(7);
OUTPUT(16) <= DATAOUT_Q(6);
OUTPUT(17) <= DATAOUT_Q(5);
OUTPUT(18) <= DATAOUT_Q(4);
OUTPUT(19) <= DATAOUT_Q(3);
OUTPUT(20) <= DATAOUT_Q(2);
--ODDR2_002 : ODDR2
--generic map(
--	DDR_ALIGNMENT => "NONE", -- Sets output alignment to "NONE", "C0", "C1" 
--	INIT => '0', -- Sets initial state of the Q output to '0' or '1'
--	SRTYPE => "SYNC") -- Specifies "SYNC" or "ASYNC" set/reset
--port map (
--	Q => OUTPUT(21), -- 1-bit output data
--	C0 => CLKOUT, -- 1-bit clock input
--	C1 => not CLKOUT, -- 1-bit clock input
--	CE => '1',  -- 1-bit clock enable input
--	D0 => '0',   -- 1-bit data input (associated with C0)
--	D1 => '1',   -- 1-bit data input (associated with C1)
--	R => '0',    -- 1-bit reset input
--	S => '1'     -- 1-bit set input
--);
OUTPUT(21) <= not CLK_P;	-- invert for better timing
OUTPUT(22) <= DATAOUT_I(1);
OUTPUT(23) <= DATAOUT_I(0);
OUTPUT(24) <= DATAOUT_Q(1);
OUTPUT(25) <= DATAOUT_Q(0);

--// TEST POINTS --------------------------------------------------
--OUTPUT(26) <= 
--OUTPUT(27) <= 
--OUTPUT(28) <= 
--OUTPUT(29) <= 
--OUTPUT(30) <= 
	
--// CONTROLS -----------------------------------------------------
-- reclock multi-byte controls to avoid glitches
CONTROLS_GEN_001: process(CLK_P, REG70, REG71, REG107, REG108)
begin
	if rising_edge(CLK_P) then
		if(SIGNAL_SCALING_CHANGE_PULSE = '1') then
			SIGNAL_SCALING <= "0" & REG71 & REG70 & "0";
		end if;
	
		if (CONFIG_CHANGE_PULSE = '1') then
			-- latch in CIC decimation ratio R
			CIC_R <= "00" & REG108(5 downto 0) & REG107;	-- decimation by R
					-- decimation filter: decimation
					-- Set R = 1 to bypass. 0 is illegal
					-- Otherwise, nominal range is 1 to 16384.
		end if;
	end if;
end process;
NCO_FREQ <= REG103 & REG102 & REG101 & REG100;	
ADC_PWDN <= '0'; 	-- power down, active high
INT_OSC_EN <= '1' when REG6(1) = '0' else '0';	
	-- by default, use Y3 100 MHz oscillator. Better phase noise than the FPGA PLL.
	-- otherwise use external ADC sampling clock

-- uncomment when ADC sampling clock is generated in FPGA 
--ODDR2_003 : ODDR2
--generic map(
--	DDR_ALIGNMENT => "NONE", -- Sets output alignment to "NONE", "C0", "C1" 
--	INIT => '0', -- Sets initial state of the Q output to '0' or '1'
--	SRTYPE => "SYNC") -- Specifies "SYNC" or "ASYNC" set/reset
--port map (
--	Q => ADC_CLK, -- 1-bit output data
--	C0 => CLK_P, -- 1-bit clock input
--	C1 => not CLK_P, -- 1-bit clock input
--	CE => '1',  -- 1-bit clock enable input
--	D0 => '0',   -- 1-bit data input (associated with C0)
--	D1 => '1',   -- 1-bit data input (associated with C1)
--	R => '0',    -- 1-bit reset input
--	S => '1'     -- 1-bit set input
--);


SREG0(0) <= PLL_STROBE;



	
--// COMSCOPE -----------------------------------------------------
--  Comment in/out as needed 
-- 
-- Define Comscope traces here 
-- Trace 1
S1_1 <= DATA8_I(11 downto 4);
S1_1_CLK <= '1';

-- Trace 2
S2_1 <= DATA8_Q(11 downto 4);
S2_1_CLK <= '1';

-- Trace 3
S3_1 <= ADC_DATA_IN_D(13 downto 6);
S3_1_CLK <= '1';

-- Trigger (1-bit format, undefine

S1_1B_CLK <= S1_1_CLK or REG240(7);	-- Nominal sampling clock or real-time processing clock
S1_2B_CLK <= S1_2_CLK or REG240(7);	-- Nominal sampling clock or real-time processing clock
S1_3B_CLK <= S1_3_CLK or REG240(7);	-- Nominal sampling clock or real-time processing clock
S1_4B_CLK <= S1_4_CLK or REG240(7);	-- Nominal sampling clock or real-time processing clock

S2_1B_CLK <= S2_1_CLK or REG242(7);	-- Nominal sampling clock or real-time processing clock
S2_2B_CLK <= S2_2_CLK or REG242(7);	-- Nominal sampling clock or real-time processing clock
S2_3B_CLK <= S2_3_CLK or REG242(7);	-- Nominal sampling clock or real-time processing clock
S2_4B_CLK <= S2_4_CLK or REG242(7);	-- Nominal sampling clock or real-time processing clock

S3_1B_CLK <= S3_1_CLK or REG244(7);	-- Nominal sampling clock or real-time processing clock
S3_2B_CLK <= S3_2_CLK or REG244(7);	-- Nominal sampling clock or real-time processing clock
S3_3B_CLK <= S3_3_CLK or REG244(7);	-- Nominal sampling clock or real-time processing clock
S3_4B_CLK <= S3_4_CLK or REG244(7);	-- Nominal sampling clock or real-time processing clock


COMSCOPE_001: COMSCOPE 
generic map(
	TRACE1_ENABLED => '1',
	TRACE2_ENABLED => '1',
	TRACE3_ENABLED => '1',
	TRACE4_ENABLED => '0'
)
port map(
   CLK => CLK_P,
	ASYNC_RESET => ASYNC_RESET,
	SIGNED_REPRESENTATION_U_SIGNED_N => REG248(6),	-- trigger format signed(1) or unsigned(0)
	REG237 => REG237,
	REG238 => REG238,
	REG239 => REG239,
	REG240 => REG240,
	REG241 => REG241,
	REG242 => REG242,
	REG243 => REG243,
	REG244 => REG244,
	REG245 => REG245,
	REG246 => REG246,
	REG247 => REG247,
	REG248 => REG248,
	REG249 => REG249,
	SIGNAL_1_BIT_1_1 => '0',
	SIGNAL_1_BIT_SAMPLE_CLK_1_1 => '0',
	SIGNAL_1_BIT_1_2 => '0',
	SIGNAL_1_BIT_SAMPLE_CLK_1_2 => '0',
	SIGNAL_1_BIT_1_3 => '0',
	SIGNAL_1_BIT_SAMPLE_CLK_1_3 => '0',
	SIGNAL_1_BIT_1_4 => '0',
	SIGNAL_1_BIT_SAMPLE_CLK_1_4 => '0',
	SIGNAL_2_BIT_1_1 => (others => '0'),
	SIGNAL_2_BIT_SAMPLE_CLK_1_1 => '0',
	SIGNAL_2_BIT_1_2 => (others => '0'),
	SIGNAL_2_BIT_SAMPLE_CLK_1_2 => '0',
	SIGNAL_2_BIT_1_3 => (others => '0'),
	SIGNAL_2_BIT_SAMPLE_CLK_1_3 => '0',
	SIGNAL_2_BIT_1_4 => (others => '0'),
	SIGNAL_2_BIT_SAMPLE_CLK_1_4 => '0',
	SIGNAL_4_BIT_1_1 => (others => '0'),
	SIGNAL_4_BIT_SAMPLE_CLK_1_1 => '0',
	SIGNAL_4_BIT_1_2 => (others => '0'),
	SIGNAL_4_BIT_SAMPLE_CLK_1_2 => '0',
	SIGNAL_4_BIT_1_3 => (others => '0'),
	SIGNAL_4_BIT_SAMPLE_CLK_1_3 => '0',
	SIGNAL_4_BIT_1_4 => (others => '0'),
	SIGNAL_4_BIT_SAMPLE_CLK_1_4 => '0',
	SIGNAL_8_BIT_1_1 => S1_1,
	SIGNAL_8_BIT_SAMPLE_CLK_1_1 => S1_1B_CLK,
	SIGNAL_8_BIT_1_2 => S1_2,
	SIGNAL_8_BIT_SAMPLE_CLK_1_2 => S1_2B_CLK,
	SIGNAL_8_BIT_1_3 => S1_3,
	SIGNAL_8_BIT_SAMPLE_CLK_1_3 => S1_3B_CLK,
	SIGNAL_8_BIT_1_4 => S1_4,
	SIGNAL_8_BIT_SAMPLE_CLK_1_4 => S1_4B_CLK,
	SIGNAL_16_BIT_1_1 => (others => '0'),
	SIGNAL_16_BIT_SAMPLE_CLK_1_1 => '0',
	SIGNAL_16_BIT_1_2 => (others => '0'),
	SIGNAL_16_BIT_SAMPLE_CLK_1_2 => '0',
	SIGNAL_16_BIT_1_3 => (others => '0'),
	SIGNAL_16_BIT_SAMPLE_CLK_1_3 => '0',
	SIGNAL_16_BIT_1_4 => (others => '0'),
	SIGNAL_16_BIT_SAMPLE_CLK_1_4 => '0',
	SIGNAL_1_BIT_2_1 => '0',
	SIGNAL_1_BIT_SAMPLE_CLK_2_1 => '0',
	SIGNAL_1_BIT_2_2 => '0',
	SIGNAL_1_BIT_SAMPLE_CLK_2_2 => '0',
	SIGNAL_1_BIT_2_3 => '0',
	SIGNAL_1_BIT_SAMPLE_CLK_2_3 => '0',
	SIGNAL_1_BIT_2_4 => '0',
	SIGNAL_1_BIT_SAMPLE_CLK_2_4 => '0',
	SIGNAL_2_BIT_2_1 => (others => '0'),
	SIGNAL_2_BIT_SAMPLE_CLK_2_1 => '0',
	SIGNAL_2_BIT_2_2 => (others => '0'),
	SIGNAL_2_BIT_SAMPLE_CLK_2_2 => '0',
	SIGNAL_2_BIT_2_3 => (others => '0'),
	SIGNAL_2_BIT_SAMPLE_CLK_2_3 => '0',
	SIGNAL_2_BIT_2_4 => (others => '0'),
	SIGNAL_2_BIT_SAMPLE_CLK_2_4 => '0',
	SIGNAL_4_BIT_2_1 => (others => '0'),
	SIGNAL_4_BIT_SAMPLE_CLK_2_1 => '0',
	SIGNAL_4_BIT_2_2 => (others => '0'),
	SIGNAL_4_BIT_SAMPLE_CLK_2_2 => '0',
	SIGNAL_4_BIT_2_3 => (others => '0'),
	SIGNAL_4_BIT_SAMPLE_CLK_2_3 => '0',
	SIGNAL_4_BIT_2_4 => (others => '0'),
	SIGNAL_4_BIT_SAMPLE_CLK_2_4 => '0',
	SIGNAL_8_BIT_2_1 => S2_1,
	SIGNAL_8_BIT_SAMPLE_CLK_2_1 => S2_1B_CLK,
	SIGNAL_8_BIT_2_2 => S2_2,
	SIGNAL_8_BIT_SAMPLE_CLK_2_2 => S2_2B_CLK,
	SIGNAL_8_BIT_2_3 => S2_3,
	SIGNAL_8_BIT_SAMPLE_CLK_2_3 => S2_3B_CLK,
	SIGNAL_8_BIT_2_4 => S2_4,
	SIGNAL_8_BIT_SAMPLE_CLK_2_4 => S2_4B_CLK,
	SIGNAL_16_BIT_2_1 => (others => '0'),
	SIGNAL_16_BIT_SAMPLE_CLK_2_1 => '0',
	SIGNAL_16_BIT_2_2 => (others => '0'),
	SIGNAL_16_BIT_SAMPLE_CLK_2_2 => '0',
	SIGNAL_16_BIT_2_3 => (others => '0'),
	SIGNAL_16_BIT_SAMPLE_CLK_2_3 => '0',
	SIGNAL_16_BIT_2_4 => (others => '0'),
	SIGNAL_16_BIT_SAMPLE_CLK_2_4 => '0',
	SIGNAL_1_BIT_3_1 => '0',
	SIGNAL_1_BIT_SAMPLE_CLK_3_1 => '0',
	SIGNAL_1_BIT_3_2 => '0',
	SIGNAL_1_BIT_SAMPLE_CLK_3_2 => '0',
	SIGNAL_1_BIT_3_3 => '0',
	SIGNAL_1_BIT_SAMPLE_CLK_3_3 => '0',
	SIGNAL_1_BIT_3_4 => '0',
	SIGNAL_1_BIT_SAMPLE_CLK_3_4 => '0',
	SIGNAL_2_BIT_3_1 => (others => '0'),
	SIGNAL_2_BIT_SAMPLE_CLK_3_1 => '0',
	SIGNAL_2_BIT_3_2 => (others => '0'),
	SIGNAL_2_BIT_SAMPLE_CLK_3_2 => '0',
	SIGNAL_2_BIT_3_3 => (others => '0'),
	SIGNAL_2_BIT_SAMPLE_CLK_3_3 => '0',
	SIGNAL_2_BIT_3_4 => (others => '0'),
	SIGNAL_2_BIT_SAMPLE_CLK_3_4 => '0',
	SIGNAL_4_BIT_3_1 => (others => '0'),
	SIGNAL_4_BIT_SAMPLE_CLK_3_1 => '0',
	SIGNAL_4_BIT_3_2 => (others => '0'),
	SIGNAL_4_BIT_SAMPLE_CLK_3_2 => '0',
	SIGNAL_4_BIT_3_3 => (others => '0'),
	SIGNAL_4_BIT_SAMPLE_CLK_3_3 => '0',
	SIGNAL_4_BIT_3_4 => (others => '0'),
	SIGNAL_4_BIT_SAMPLE_CLK_3_4 => '0',
	SIGNAL_8_BIT_3_1 => S3_1,
	SIGNAL_8_BIT_SAMPLE_CLK_3_1 => S3_1B_CLK,
	SIGNAL_8_BIT_3_2 => S3_2,
	SIGNAL_8_BIT_SAMPLE_CLK_3_2 => S3_2B_CLK,
	SIGNAL_8_BIT_3_3 => (others => '0'),
	SIGNAL_8_BIT_SAMPLE_CLK_3_3 => '0',
	SIGNAL_8_BIT_3_4 => (others => '0'),
	SIGNAL_8_BIT_SAMPLE_CLK_3_4 => '0',
	SIGNAL_16_BIT_3_1 => (others => '0'),
	SIGNAL_16_BIT_SAMPLE_CLK_3_1 => '0',
	SIGNAL_16_BIT_3_2 => (others => '0'),
	SIGNAL_16_BIT_SAMPLE_CLK_3_2 => '0',
	SIGNAL_16_BIT_3_3 => (others => '0'),
	SIGNAL_16_BIT_SAMPLE_CLK_3_3 => '0',
	SIGNAL_16_BIT_3_4 => (others => '0'),
	SIGNAL_16_BIT_SAMPLE_CLK_3_4 => '0',
	SIGNAL_1_BIT_4_1 => '0',
	SIGNAL_1_BIT_SAMPLE_CLK_4_1 => '0',
	SIGNAL_1_BIT_4_2 => '0',
	SIGNAL_1_BIT_SAMPLE_CLK_4_2 => '0',
	SIGNAL_1_BIT_4_3 => '0',
	SIGNAL_1_BIT_SAMPLE_CLK_4_3 => '0',
	SIGNAL_1_BIT_4_4 => '0',
	SIGNAL_1_BIT_SAMPLE_CLK_4_4 => '0',
	SIGNAL_2_BIT_4_1 => (others => '0'),
	SIGNAL_2_BIT_SAMPLE_CLK_4_1 => '0',
	SIGNAL_2_BIT_4_2 => (others => '0'),
	SIGNAL_2_BIT_SAMPLE_CLK_4_2 => '0',
	SIGNAL_2_BIT_4_3 => (others => '0'),
	SIGNAL_2_BIT_SAMPLE_CLK_4_3 => '0',
	SIGNAL_2_BIT_4_4 => (others => '0'),
	SIGNAL_2_BIT_SAMPLE_CLK_4_4 => '0',
	SIGNAL_4_BIT_4_1 => (others => '0'),
	SIGNAL_4_BIT_SAMPLE_CLK_4_1 => '0',
	SIGNAL_4_BIT_4_2 => (others => '0'),
	SIGNAL_4_BIT_SAMPLE_CLK_4_2 => '0',
	SIGNAL_4_BIT_4_3 => (others => '0'),
	SIGNAL_4_BIT_SAMPLE_CLK_4_3 => '0',
	SIGNAL_4_BIT_4_4 => (others => '0'),
	SIGNAL_4_BIT_SAMPLE_CLK_4_4 => '0',
	SIGNAL_8_BIT_4_1 => (others => '0'),
	SIGNAL_8_BIT_SAMPLE_CLK_4_1 => '0',
	SIGNAL_8_BIT_4_2 => (others => '0'),
	SIGNAL_8_BIT_SAMPLE_CLK_4_2 => '0',
	SIGNAL_8_BIT_4_3 => (others => '0'),
	SIGNAL_8_BIT_SAMPLE_CLK_4_3 => '0',
	SIGNAL_8_BIT_4_4 => (others => '0'),
	SIGNAL_8_BIT_SAMPLE_CLK_4_4 => '0',
	SIGNAL_16_BIT_4_1 => (others => '0'),
	SIGNAL_16_BIT_SAMPLE_CLK_4_1 => '0',
	SIGNAL_16_BIT_4_2 => (others => '0'),
	SIGNAL_16_BIT_SAMPLE_CLK_4_2 => '0',
	SIGNAL_16_BIT_4_3 => (others => '0'),
	SIGNAL_16_BIT_SAMPLE_CLK_4_3 => '0',
	SIGNAL_16_BIT_4_4 => (others => '0'),
	SIGNAL_16_BIT_SAMPLE_CLK_4_4 => '0',
	TRIGGER_1_BIT_1 => T1,
	TRIGGER_1_BIT_SAMPLE_CLK_1 => T1_CLK,
	TRIGGER_1_BIT_2 => T2,
	TRIGGER_1_BIT_SAMPLE_CLK_2 => T2_CLK,
	TRIGGER_1_BIT_3 => '0',
	TRIGGER_1_BIT_SAMPLE_CLK_3 => '0',
	TRIGGER_1_BIT_4 => '0',
	TRIGGER_1_BIT_SAMPLE_CLK_4 => '0',
	TRIGGER_2_BIT_1 => (others => '0'),
	TRIGGER_2_BIT_SAMPLE_CLK_1 => '0',
	TRIGGER_2_BIT_2 => (others => '0'),
	TRIGGER_2_BIT_SAMPLE_CLK_2 => '0',
	TRIGGER_2_BIT_3 => (others => '0'),
	TRIGGER_2_BIT_SAMPLE_CLK_3 => '0',
	TRIGGER_2_BIT_4 => (others => '0'),
	TRIGGER_2_BIT_SAMPLE_CLK_4 => '0',
	TRIGGER_4_BIT_1 => (others => '0'),
	TRIGGER_4_BIT_SAMPLE_CLK_1 => '0',
	TRIGGER_4_BIT_2 => (others => '0'),
	TRIGGER_4_BIT_SAMPLE_CLK_2 => '0',
	TRIGGER_4_BIT_3 => (others => '0'),
	TRIGGER_4_BIT_SAMPLE_CLK_3 => '0',
	TRIGGER_4_BIT_4 => (others => '0'),
	TRIGGER_4_BIT_SAMPLE_CLK_4 => '0',
	TRIGGER_8_BIT_1 => (others => '0'),
	TRIGGER_8_BIT_SAMPLE_CLK_1 => '0',
	TRIGGER_8_BIT_2 => (others => '0'),
	TRIGGER_8_BIT_SAMPLE_CLK_2 => '0',
	TRIGGER_8_BIT_3 => (others => '0'),
	TRIGGER_8_BIT_SAMPLE_CLK_3 => '0',
	TRIGGER_8_BIT_4 => (others => '0'),
	TRIGGER_8_BIT_SAMPLE_CLK_4 => '0',
	TRIGGER_16_BIT_1 => (others => '0'),
	TRIGGER_16_BIT_SAMPLE_CLK_1 => '0',
	TRIGGER_16_BIT_2 => (others => '0'),
	TRIGGER_16_BIT_SAMPLE_CLK_2 => '0',
	TRIGGER_16_BIT_3 => (others => '0'),
	TRIGGER_16_BIT_SAMPLE_CLK_3 => '0',
	TRIGGER_16_BIT_4 => (others => '0'),
	TRIGGER_16_BIT_SAMPLE_CLK_4 => '0',
	REG250 => REG250,
	REG251 => REG251,
	TRIGGER_REARM_TOGGLE => TRIGGER_REARM_TOGGLE,
	FORCE_TRIGGER_TOGGLE => FORCE_TRIGGER_TOGGLE,
	SREG250_READ_TOGGLE => SREG250_READ_TOGGLE,
	START_CAPTURE_TOGGLE => START_CAPTURE_TOGGLE,
	TP => open
);



--// ARM CO-PROCESSOR INTERFACE ---------------------------------------------------
-- Primarily used for monitoring and control.
-- 8-bit shared address/data bus. clock synchronous.
-- PART OF THE FRAMEWORK. DO NOT CHANGE unless you want to 
-- add more control registers (see UC_WRITE_001) or status registers (see UC_READ_001).
-- 
-- Address latch
UC_ADDR_001: process(UC_CSIB_IN, UC_WEBG, UC_ALE_IN, UC_AD)
begin
	if rising_edge(UC_WEBG) then
		if (UC_CSIB_IN = '0') and (UC_ALE_IN = '1') then
			UC_ADDRESS <= UC_AD;
	  	end if;
  	end if;
end process;

-- Read status registers 
-- This template defines 10 status registers. 
-- User can define additional control registers using the same pattern. 
UC_READ_001: process(UC_ADDRESS, REG251, REG250, 
	SREG0, SREG1, SREG2, SREG3, SREG4, SREG5, SREG6, SREG7, SREG8, SREG9, SREG10)
begin
	case(UC_ADDRESS) is
		-- ComBlock-reserved addresses ----
		when "11111101" => UC_AD_LOCAL <= REVISION;	-- reg 253 = fpga revision
		when "11111100" => UC_AD_LOCAL <= OPTION;		-- reg 252 = fpga option
		when "11111011" => UC_AD_LOCAL <= REG251;		-- reg 251 =comscope  status
		when "11111010" => UC_AD_LOCAL <= REG250;		-- reg 250 = comscope data byte
		
		-- User-defined status registers -------
		when "00000000" => UC_AD_LOCAL <= SREG0;
		when "00000001" => UC_AD_LOCAL <= SREG1;
		when "00000010" => UC_AD_LOCAL <= SREG2;
		when "00000011" => UC_AD_LOCAL <= SREG3;
		when "00000100" => UC_AD_LOCAL <= SREG4;
		when "00000101" => UC_AD_LOCAL <= SREG5;
		when "00000110" => UC_AD_LOCAL <= SREG6;
		when "00000111" => UC_AD_LOCAL <= SREG7;
		when "00001000" => UC_AD_LOCAL <= SREG8;
		when "00001001" => UC_AD_LOCAL <= SREG9;
		--when "00001010" => UC_AD_LOCAL <= SREG10;
		when others => UC_AD_LOCAL <= SREG10;
		-- user can add more status registers as needed (200+)
	end case;
end process;

UC_AD <= UC_AD_LOCAL when (UC_CSIB_IN = '0') and (UC_REB_IN = '0') else (others => 'Z');
	-- when to drive the data bus between ARM and FPGA
	
-- toggle a few flags upon reading specific status registers
UC_READ_002: process(ASYNC_RESET, UC_REB_IN, UC_CSIB_IN, UC_ADDRESS, 
						SREG250_READ_TOGGLE, SREG254_READ_TOGGLE)
begin
	if(ASYNC_RESET = '1') then
		SREG250_READ_TOGGLE <= '0';
		SREG254_READ_TOGGLE <= '0';
	elsif rising_edge(UC_REB_IN) then
		if (UC_CSIB_IN = '0')  then
			if (UC_ADDRESS = 250) then
				SREG250_READ_TOGGLE <= not SREG250_READ_TOGGLE;
			end if;
			if (UC_ADDRESS = 254) then
				SREG254_READ_TOGGLE <= not SREG254_READ_TOGGLE;
			end if;
			if(UC_ADDRESS = 7) then
				-- latch in frequency error to prevent changes while reading the 4 status registers.
				SREG8 <= CLK_P_FREQ(15 downto 8);
				SREG9 <= CLK_P_FREQ(23 downto 16);
				SREG10 <= "00000" & CLK_P_FREQ(26 downto 24);
			end if;
		end if;
	end if;
end process;
			
	
-- write to FPGA control registers
-- User can define additional control registers using the same pattern. 
UC_WRITE_001: process(UC_CSIB_IN, UC_WEBG, UC_ALE_IN, UC_REB_IN, UC_AD)
begin
	if rising_edge(UC_WEBG) then
		if (UC_CSIB_IN = '0') and (UC_ALE_IN = '0') and (UC_REB_IN = '1')  then
			-- These registers are passed through by the ARM processor.
			-- Their definition matches the user's manual
			if (UC_ADDRESS = 6) then
				REG6 <= UC_AD;
			end if;
			if (UC_ADDRESS = 70) then
				REG70 <= UC_AD;
			end if;
			if (UC_ADDRESS = 71) then
				REG71 <= UC_AD;
				SIGNAL_SCALING_CHANGE_TOGGLE <= not SIGNAL_SCALING_CHANGE_TOGGLE;
			end if;
			-- These registers are pre-processed by the ARM processor (change in format, etc)
			-- Their definition is not in the user's manual.
			if (UC_ADDRESS = 100) then
				REG100 <= UC_AD;
			end if;
			if (UC_ADDRESS = 101) then
				REG101 <= UC_AD;
			end if;
			if (UC_ADDRESS = 102) then
				REG102 <= UC_AD;
			end if;
			if (UC_ADDRESS = 103) then
				REG103 <= UC_AD;
			end if;
			if (UC_ADDRESS = 107) then
				REG107 <= UC_AD;
			end if;
			if (UC_ADDRESS = 108) then
				REG108 <= UC_AD;
				CONFIG_CHANGE_TOGGLE <= not CONFIG_CHANGE_TOGGLE;	
				-- one MUST reset the CIC filter after changing the decimation
			end if;
			-- user can add more control registers as needed (200+)

		end if;
	end if;
end process;

--// reserved COMSCOPE -------------------------------------------------------
-- Comment out if ComScope feature is unused
UC_WRITE_002: process(UC_CSIB_IN, UC_WEBG, UC_ADDRESS, UC_AD, 
	FORCE_TRIGGER_TOGGLE, TRIGGER_REARM_TOGGLE, START_CAPTURE_TOGGLE)
begin
	if rising_edge(UC_WEBG) then
		if (UC_CSIB_IN = '0') and (UC_ALE_IN = '0') and (UC_REB_IN = '1')  then
			-- Reserved for ComScope			
			if (UC_ADDRESS = 237) then
				REG237 <= UC_AD;	
			end if;
			if (UC_ADDRESS = 238) then
				REG238 <= UC_AD;	
			end if;
			if (UC_ADDRESS = 239) then
				REG239 <= UC_AD;
			end if;
			if (UC_ADDRESS = 240) then
				REG240 <= UC_AD;	
			end if;
			if (UC_ADDRESS = 241) then
				REG241 <= UC_AD;	
			end if;
			if (UC_ADDRESS = 242) then
				REG242 <= UC_AD;	
			end if;
			if (UC_ADDRESS = 243) then
				REG243 <= UC_AD;	
			end if;
			if (UC_ADDRESS = 244) then
				REG244 <= UC_AD;	
			end if;
			if (UC_ADDRESS = 245) then
				REG245 <= UC_AD;	
			end if;
			if (UC_ADDRESS = 246) then
				REG246 <= UC_AD;	
			end if;
			if (UC_ADDRESS = 247) then
				REG247 <= UC_AD;	
			end if;
			if (UC_ADDRESS = 248) then
				REG248 <= UC_AD;	
			end if;
			if (UC_ADDRESS = 249) then
				REG249 <= UC_AD;	
			end if;
			if (UC_ADDRESS = 250) then 
				-- detect one-shot force trigger	
				if(UC_AD(0) = '1') then	
					FORCE_TRIGGER_TOGGLE <= not(FORCE_TRIGGER_TOGGLE);
				end if;	
				-- detect one-shot re-arm	
				if(UC_AD(1) = '1') then	
					TRIGGER_REARM_TOGGLE <= not(TRIGGER_REARM_TOGGLE);
				end if;	
				-- start capturing again
				if(UC_AD(2) = '1') then	
					START_CAPTURE_TOGGLE <= not (START_CAPTURE_TOGGLE);
				end if;	
			end if;
		end if;
	end if;
end process;

--// end of COMSCOPE section -------------------------------------------------------

-- Reclock CONFIG_CHANGE_TOGGLE to be synchronous with processing clock CLK_P
RECLOCK_002: process(CLK_P)
begin
	if rising_edge(CLK_P) then
		CONFIG_CHANGE_TOGGLE_D <= CONFIG_CHANGE_TOGGLE;
		CONFIG_CHANGE_TOGGLE_D2 <= CONFIG_CHANGE_TOGGLE_D;
		CONFIG_CHANGE_PULSE_D <= CONFIG_CHANGE_PULSE;
		CONFIG_CHANGE_PULSE_D2 <= CONFIG_CHANGE_PULSE_D;
		
		if(CONFIG_CHANGE_TOGGLE_D /= CONFIG_CHANGE_TOGGLE_D2) then
			CONFIG_CHANGE_PULSE <= '1';
		else
			CONFIG_CHANGE_PULSE <= '0';
		end if;
	end if;
end process;

RECLOCK_003: process(CLK_P)
begin
	if rising_edge(CLK_P) then
		SIGNAL_SCALING_CHANGE_TOGGLE_D <= SIGNAL_SCALING_CHANGE_TOGGLE;
		SIGNAL_SCALING_CHANGE_TOGGLE_D2 <= SIGNAL_SCALING_CHANGE_TOGGLE_D;
		if(SIGNAL_SCALING_CHANGE_TOGGLE_D /= SIGNAL_SCALING_CHANGE_TOGGLE_D2) then
			SIGNAL_SCALING_CHANGE_PULSE <= '1';
		else
			SIGNAL_SCALING_CHANGE_PULSE <= '0';
		end if;
	end if;
end process;



end Behavioral;
